@@ -7,12 +7,10 @@
 #ifndef LIBBOARDGAME_BASE_ENGINE_H
 #define LIBBOARDGAME_BASE_ENGINE_H
 
-#include <memory>
 #include "libboardgame_gtp/Engine.h"
 
 namespace libboardgame_base {
 
-using namespace std;
 using libboardgame_gtp::Arguments;
 using libboardgame_gtp::Response;
 
@@ -23,21 +21,14 @@ class Engine
 {
 public:
     void cmd_cputime(Response&);
-    void cmd_cputime_diff(Response&);
-    void cmd_interrupt();
     void cmd_set_random_seed(const Arguments&);
 
     Engine();
 
     ~Engine();
 
-    void interrupt() override;
-
 protected:
     void on_handle_cmd_begin() override;
-
-private:
-    double m_cpu_time_last;
 };
 
 //-----------------------------------------------------------------------------
