@@ -25,10 +25,10 @@ using namespace libboardgame_util;
     Geometry. The convention for the coordinates is that the top left corner of
     the board has the coordinates (0,0). Point::null() has the meaning
     "no point".
-    @tparam M The maximum number of on-board points of the geometries this point
-    is used in (excluding the null point).
-    @tparam W The maximum width of the geometries this point is used in.
-    @tparam H The maximum height of the geometries this point is used in.
+    @tparam M The maximum number of on-board points of all geometries this
+    point is used in (excluding the null point).
+    @tparam W The maximum width of all geometries this point is used in.
+    @tparam H The maximum height of all geometries this point is used in.
     @tparam I An unsigned integer type to store the point value
     @tparam S A class with functions to convert points from and to strings
     depending on the string representation of points in the game. */
@@ -52,8 +52,6 @@ public:
 
     static_assert(max_onboard <= max_width * max_height, "");
 
-    static const unsigned begin_onboard = 0;
-
     static const unsigned range_onboard = max_onboard;
 
     static const unsigned range = max_onboard + 1;
@@ -75,10 +73,6 @@ public:
     /** Return point as an integer between 0 and Point::range */
     unsigned to_int() const;
 
-    /** Return point as an integer between 0 and Point::range_onboard.
-        @pre ! is_null() */
-    unsigned to_int_not_null() const;
-
 private:
     static const IntType value_uninitialized = range;
 
@@ -152,13 +146,6 @@ inline unsigned Point<M, W, H, I, S>::to_int() const
     return m_i;
 }
 
-template<unsigned M, unsigned W, unsigned H, typename I, class S>
-inline unsigned Point<M, W, H, I, S>::to_int_not_null() const
-{
-    LIBBOARDGAME_ASSERT(! is_null());
-    return m_i;
-}
-
 //-----------------------------------------------------------------------------
 
 } // namespace libboardgame_base
