@@ -9,13 +9,11 @@
 
 #include <algorithm>
 #include <cassert>
-#include <cstddef>
 #include <sstream>
 #include <string>
 #include <iterator>
 #include <vector>
 #include "CmdLineRange.h"
-#include "Failure.h"
 
 namespace libboardgame_gtp {
 
@@ -37,7 +35,7 @@ public:
 
     /** Construct with a command line.
         @see init() */
-    CmdLine(const string& line);
+    explicit CmdLine(const string& line);
 
     ~CmdLine();
 
