@@ -10,30 +10,25 @@
 
 #include "Response.h"
 
-#include <iostream>
-
 namespace libboardgame_gtp {
 
-using namespace std;
-
 //-----------------------------------------------------------------------------
 
 ostringstream Response::s_dummy;
 
 Response::~Response()
-{
-}
+{ }
 
 void Response::write(ostream& out, string& buffer) const
 {
     buffer = m_stream.str();
     bool was_newline = false;
-    for (auto i = buffer.begin(); i != buffer.end(); ++i)
+    for (auto c : buffer)
     {
-        bool is_newline =(*i == '\n');
+        bool is_newline = (c == '\n');
         if (is_newline && was_newline)
             out << ' ';
-        out << *i;
+        out << c;
         was_newline = is_newline;
     }
     if (! was_newline)
