@@ -86,11 +86,22 @@ public:
 
     void link_children(NodeIdx first_child, unsigned short nu_children);
 
+    /** Faster version of link_children() for single-threaded parts of the
+        code. */
+    void link_children_st(NodeIdx first_child, unsigned short nu_children);
+
     void unlink_children();
 
+    /** Faster version of unlink_children() for single-threaded parts of the
+        code. */
+    void unlink_children_st();
+
     void add_value(Float v, Float weight = 1);
 
-    void remove_value(Float v, Float weight = 1);
+    /** Add a value with weight 1 and remove a previously added loss.
+        Needed for the implementation of virtual losses in multi-threaded
+        MCTS and more efficient that a separate add and remove call. */
+    void add_value_remove_loss(Float v);
 
     void inc_visit_count();
 
@@ -125,6 +136,19 @@ void Node<M, F, MT>::add_value(Float v, Float weight)
     m_value_count.store(count, memory_order_relaxed);
 }
 
+template<typename M, typename F, bool MT>
+void Node<M, F, MT>::add_value_remove_loss(Float v)
+{
+    // Intentionally uses no synchronization and does not care about
+    // lost updates in multi-threaded mode
+    Float count = m_value_count.load(memory_order_relaxed);
+    if (count == 0)
+        return; // Adding the virtual loss was a lost update
+    Float value = m_value.load(memory_order_relaxed);
+    value += v / count;
+    m_value.store(value, memory_order_relaxed);
+}
+
 template<typename M, typename F, bool MT>
 void Node<M, F, MT>::copy_data_from(const Node& node)
 {
@@ -141,9 +165,14 @@ void Node<M, F, MT>::copy_data_from(const Node& node)
     static_assert(sizeof(Node) == sizeof(Dummy), "");
 
     m_move = node.m_move;
-    m_value_count = node.m_value_count.load();
-    m_value = node.m_value.load();
-    m_visit_count = node.m_visit_count.load();
+    // Load/store relaxed (it wouldn't even need to be atomic) because this
+    // function is only used before the multi-threaded search.
+    m_value_count.store(node.m_value_count.load(memory_order_relaxed),
+                        memory_order_relaxed);
+    m_value.store(node.m_value.load(memory_order_relaxed),
+                  memory_order_relaxed);
+    m_visit_count.store(node.m_visit_count.load(memory_order_relaxed),
+                        memory_order_relaxed);
 }
 
 template<typename M, typename F, bool MT>
@@ -233,21 +262,15 @@ inline void Node<M, F, MT>::link_children(NodeIdx first_child,
 }
 
 template<typename M, typename F, bool MT>
-void Node<M, F, MT>::remove_value(Float v, Float weight)
+inline void Node<M, F, MT>::link_children_st(NodeIdx first_child,
+                                             unsigned short nu_children)
 {
-    // Intentionally uses no synchronization and does not care about
-    // lost updates in multi-threaded mode
-    Float count = m_value_count.load(memory_order_relaxed);
-    if (count > weight)
-    {
-        Float value = m_value.load(memory_order_relaxed);
-        count -= weight;
-        value -= weight * (v - value) / count;
-        m_value.store(value, memory_order_relaxed);
-        m_value_count.store(count, memory_order_relaxed);
-    }
-    else
-        m_value_count.store(0, memory_order_relaxed);
+    LIBBOARDGAME_ASSERT(nu_children < Move::range);
+    // first_child cannot be 0 because 0 is always used for the root node
+    LIBBOARDGAME_ASSERT(first_child != 0);
+    m_first_child = first_child;
+    // Store relaxed (wouldn't even need to be atomic)
+    m_nu_children.store(nu_children, memory_order_relaxed);
 }
 
 template<typename M, typename F, bool MT>
@@ -256,6 +279,13 @@ inline void Node<M, F, MT>::unlink_children()
     m_nu_children.store(0, memory_order_release);
 }
 
+template<typename M, typename F, bool MT>
+inline void Node<M, F, MT>::unlink_children_st()
+{
+    // Store relaxed (wouldn't even need to be atomic)
+    m_nu_children.store(0, memory_order_relaxed);
+}
+
 //-----------------------------------------------------------------------------
 
 } // namespace libboardgame_mcts
