@@ -7,6 +7,8 @@
 #ifndef LIBBOARDGAME_MCTS_PLAYER_MOVE_H
 #define LIBBOARDGAME_MCTS_PLAYER_MOVE_H
 
+#include <cstdint>
+
 namespace libboardgame_mcts {
 
 //-----------------------------------------------------------------------------
@@ -22,13 +24,6 @@ struct PlayerMove
 
     MOVE move;
 
-    static PlayerMove null()
-    {
-        PlayerMove m;
-        m.move = MOVE::null();
-        return m;
-    }
-
     PlayerMove() = default;
 
     PlayerMove(PlayerInt player, MOVE move)
