@@ -18,8 +18,6 @@
 
 namespace libboardgame_sgf {
 
-using namespace std;
-
 //-----------------------------------------------------------------------------
 
 namespace {
@@ -35,21 +33,11 @@ bool is_ascii_space(int c)
 
 //-----------------------------------------------------------------------------
 
-Reader::ReadError::ReadError(const string& s)
-    : Exception(s)
-{
-}
-
-//-----------------------------------------------------------------------------
-
 Reader::Reader()
-  : m_read_only_main_variation(false)
-{
-}
+{ }
 
 Reader::~Reader()
-{
-}
+{ }
 
 void Reader::consume_char(char expected)
 {
@@ -115,7 +103,7 @@ void Reader::read(istream& in, bool check_single_tree,
         int c = m_in->peek();
         if (c == EOF)
         {
-            if (more_game_trees_left != nullptr)
+            if (more_game_trees_left)
                 *more_game_trees_left = false;
             return;
         }
@@ -125,7 +113,7 @@ void Reader::read(istream& in, bool check_single_tree,
                 throw ReadError("Input has multiple game trees");
             else
             {
-                if (more_game_trees_left != nullptr)
+                if (more_game_trees_left)
                     *more_game_trees_left = true;
                 return;
             }
