@@ -18,6 +18,7 @@
 namespace libboardgame_sgf {
 
 using libboardgame_sgf::util::find_root;
+using libboardgame_util::trim;
 
 //-----------------------------------------------------------------------------
 
@@ -51,6 +52,11 @@ void SgfTree::delete_all_variations()
     }
 }
 
+double SgfTree::get_bad_move(const SgfNode& node)
+{
+    return node.parse_property<double>("BM", 0);
+}
+
 string SgfTree::get_comment(const SgfNode& node) const
 {
     return node.get_property("C", "");
@@ -67,6 +73,11 @@ string SgfTree::get_date_today()
     return date;
 }
 
+double SgfTree::get_good_move(const SgfNode& node)
+{
+    return node.parse_property<double>("TE", 0);
+}
+
 unique_ptr<SgfNode> SgfTree::get_tree_transfer_ownership()
 {
     return move(m_root);
@@ -97,6 +108,16 @@ void SgfTree::init(unique_ptr<SgfNode>& root)
     m_modified = false;
 }
 
+bool SgfTree::is_doubtful_move(const SgfNode& node)
+{
+    return node.has_property("DO");
+}
+
+bool SgfTree::is_interesting_move(const SgfNode& node)
+{
+    return node.has_property("IT");
+}
+
 void SgfTree::make_first_child(const SgfNode& node)
 {
     auto parent = node.get_parent_or_null();
