@@ -13,7 +13,6 @@
 namespace libboardgame_sgf {
 
 using namespace std;
-using libboardgame_util::trim;
 
 //-----------------------------------------------------------------------------
 
@@ -117,19 +116,19 @@ public:
 
     void remove_move_annotation(const SgfNode& node);
 
-    double get_good_move(const SgfNode& node) const;
+    static double get_good_move(const SgfNode& node);
 
     void set_good_move(const SgfNode& node, double value = 1);
 
-    double get_bad_move(const SgfNode& node) const;
+    static double get_bad_move(const SgfNode& node);
 
     void set_bad_move(const SgfNode& node, double value = 1);
 
-    bool is_doubtful_move(const SgfNode& node) const;
+    static bool is_doubtful_move(const SgfNode& node);
 
     void set_doubtful_move(const SgfNode& node);
 
-    bool is_interesting_move(const SgfNode& node) const;
+    static bool is_interesting_move(const SgfNode& node);
 
     void set_interesting_move(const SgfNode& node);
 
@@ -180,11 +179,6 @@ inline void SgfTree::clear_modified()
     m_modified = false;
 }
 
-inline double SgfTree::get_bad_move(const SgfNode& node) const
-{
-    return node.parse_property<double>("BM", 0);
-}
-
 inline string SgfTree::get_date() const
 {
     return m_root->get_property("DT", "");
@@ -195,11 +189,6 @@ inline string SgfTree::get_event() const
     return m_root->get_property("EV", "");
 }
 
-inline double SgfTree::get_good_move(const SgfNode& node) const
-{
-    return node.parse_property<double>("TE", 0);
-}
-
 inline bool SgfTree::is_modified() const
 {
     return m_modified;
@@ -220,16 +209,6 @@ inline string SgfTree::get_time() const
     return m_root->get_property("TM", "");
 }
 
-inline bool SgfTree::is_doubtful_move(const SgfNode& node) const
-{
-    return node.has_property("DO");
-}
-
-inline bool SgfTree::is_interesting_move(const SgfNode& node) const
-{
-    return node.has_property("IT");
-}
-
 inline SgfNode& SgfTree::non_const(const SgfNode& node)
 {
     LIBBOARDGAME_ASSERT(contains(node));
