@@ -12,18 +12,13 @@
 
 namespace libboardgame_sgf {
 
-using namespace std;
-
 //-----------------------------------------------------------------------------
 
 TreeReader::TreeReader()
-    : m_current(nullptr)
-{
-}
+{ }
 
 TreeReader::~TreeReader()
-{
-}
+{ }
 
 unique_ptr<SgfNode> TreeReader::get_tree_transfer_ownership()
 {
@@ -50,7 +45,7 @@ void TreeReader::on_begin_node(bool is_root)
 {
     if (is_root)
     {
-        m_root.reset(new SgfNode());
+        m_root.reset(new SgfNode);
         m_current = m_root.get();
     }
     else
