@@ -34,18 +34,18 @@ void TreeWriter::write()
 void TreeWriter::write_node(const SgfNode& node)
 {
     m_writer.begin_node();
-    for (PropertyIterator i(node); i; ++i)
-        write_property(i->id, i->values);
+    for (auto& i : node.get_properties())
+        write_property(i.id, i.values);
     m_writer.end_node();
     if (! node.has_children())
         return;
     else if (node.has_single_child())
         write_node(node.get_child());
     else
-        for (ChildIterator i(node); i; ++i)
+        for (auto& i : node.get_children())
         {
             m_writer.begin_tree();
-            write_node(*i);
+            write_node(i);
             m_writer.end_tree();
         }
 }
