@@ -36,7 +36,7 @@ public:
 
     void set_one_prop_value_per_line(bool enable);
 
-    void set_indent(unsigned indent);
+    void set_indent(int indent);
 
     /** @} */ // @name
 
@@ -61,7 +61,7 @@ inline void TreeWriter::set_one_prop_value_per_line(bool enable)
     m_writer.set_one_prop_value_per_line(enable);
 }
 
-inline void TreeWriter::set_indent(unsigned indent)
+inline void TreeWriter::set_indent(int indent)
 {
     m_writer.set_indent(indent);
 }
