@@ -7,8 +7,6 @@
 #ifndef LIBBOARDGAME_UTIL_ASSERT_H
 #define LIBBOARDGAME_UTIL_ASSERT_H
 
-#include "libboardgame_sys/Compiler.h"
-
 namespace libboardgame_util {
 
 //-----------------------------------------------------------------------------
@@ -28,12 +26,9 @@ public:
 #if LIBBOARDGAME_DEBUG
 
 /** Function used by the LIBBOARDGAME_ASSERT macro to run all assertion
-    handlers.
-    This function is marked with LIBBOARDGAME_NORETURN because it calls abort()
-    and otherwise some versions of GCC generate lots of bogus warnings (GCC
-    bug #45180). */
-void handle_assertion(const char* expression, const char* file,
-                      int line) LIBBOARDGAME_NORETURN;
+    handlers. */
+[[noreturn]] void handle_assertion(const char* expression, const char* file,
+                                   int line);
 
 #endif
 
