@@ -4,13 +4,16 @@
     @copyright GNU General Public License version 3 or later */
 //-----------------------------------------------------------------------------
 
+#if ! LIBBOARDGAME_DISABLE_LOG
+
+//-----------------------------------------------------------------------------
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include "Log.h"
 
-#include <fstream>
 #include <iostream>
 
 #if defined ANDROID || defined __ANDROID__
@@ -74,65 +77,48 @@ AndroidBuf android_buffer;
 
 #endif // defined(ANDROID) || defined(__ANDROID__)
 
-ostream* log_stream = &cerr;
-
-/** Unopened file stream serves as null stream. */
-ofstream null_stream;
-
 } // namespace
 
 //-----------------------------------------------------------------------------
 
-ostream& get_log()
-{
-    return *log_stream;
-}
+ostream* _log_stream = &cerr;
 
-bool is_log_null()
-{
-    return log_stream == &null_stream;
-}
+//-----------------------------------------------------------------------------
 
-void log(const string& s)
+void _log(const string& s)
 {
-    if (is_log_null())
+    if (! _log_stream)
         return;
     if (s.empty())
-        *log_stream << '\n';
+        *_log_stream << '\n';
     else if (s.back() == '\n')
-        *log_stream << s;
+        *_log_stream << s;
     else
     {
         string line = s;
         line += '\n';
-        *log_stream << line;
+        *_log_stream << line;
     }
 }
 
-void log_close()
+void _log_close()
 {
 #if defined ANDROID || defined __ANDROID__
     cerr.rdbuf(nullptr);
 #endif
 }
 
-void log_init()
+void _log_init()
 {
 #if defined ANDROID || defined __ANDROID__
     cerr.rdbuf(&android_buffer);
 #endif
 }
 
-void set_log(ostream& out)
-{
-    log_stream = &out;
-}
-
-void set_log_null()
-{
-    set_log(null_stream);
-}
-
 //-----------------------------------------------------------------------------
 
 } // namespace libboardgame_util
+
+//-----------------------------------------------------------------------------
+
+#endif // ! LIBBOARDGAME_DISABLE_LOG
