@@ -10,24 +10,18 @@
 
 #include "WallTimeSource.h"
 
+#include <chrono>
+
 namespace libboardgame_util {
 
 using namespace std::chrono;
 
 //-----------------------------------------------------------------------------
 
-WallTimeSource::WallTimeSource()
-{
-    m_start = system_clock::now();
-}
-
 double WallTimeSource::operator()()
 {
-    // Logically, there is no need to return the time since m_start, we could
-    // also use time_since_epoch(), but during debugging it is nicer to
-    // deal with smaller numbers.
-    auto diff = system_clock::now() - m_start;
-    return duration_cast<duration<double>>(diff).count();
+    auto t = system_clock::now().time_since_epoch();
+    return duration_cast<duration<double>>(t).count();
 }
 
 //----------------------------------------------------------------------------
