@@ -11,7 +11,7 @@
 #include "Board.h"
 
 #include <functional>
-#include "libboardgame_util/Unused.h"
+#include "MoveMarker.h"
 
 namespace libpentobi_base {
 
@@ -55,6 +55,9 @@ Board::Board(Variant variant)
     m_color_char[Color(1)] = 'O';
     m_color_char[Color(2)] = '#';
     m_color_char[Color(3)] = '@';
+    for_each_color([&](Color c) {
+        m_state_color[c].forbidden[Point::null()] = false;
+    });
     init_variant(variant);
     init();
 #if LIBBOARDGAME_DEBUG
@@ -78,6 +81,16 @@ void Board::copy_from(const Board& bd)
     }
 }
 
+const Transform* Board::find_transform(Move mv) const
+{
+    auto& geo = get_geometry();
+    auto& move_info = get_move_info(mv);
+    PiecePoints points;
+    for (Point p : move_info)
+        points.push_back(CoordPoint(geo.get_x(p), geo.get_y(p)));
+    return get_piece_info(move_info.get_piece()).find_transform(geo, points);
+}
+
 void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
 {
     moves.clear();
@@ -111,6 +124,25 @@ void Board::gen_moves(Color c, Point p, unsigned adj_status,
         }
 }
 
+unsigned Board::get_bonus(Color c) const
+{
+    if (get_pieces_left(c).size() > 0)
+        return 0;
+    unsigned bonus = m_bonus_all_pieces;
+    unsigned i = m_moves.size();
+    while (i > 0)
+    {
+        --i;
+        if (m_moves[i].color == c)
+        {
+            if (get_move_info(m_moves[i].move).size() == 1)
+                bonus += m_bonus_one_piece;
+            break;
+        }
+    }
+    return bonus;
+}
+
 Color Board::get_effective_to_play() const
 {
     Color c = m_state_base.to_play;
@@ -281,6 +313,11 @@ void Board::init_variant(Variant variant)
         m_bonus_all_pieces = 0;
         m_bonus_one_piece = 0;
     }
+    else if (m_variant == Variant::nexos || m_variant == Variant::nexos_2)
+    {
+        m_bonus_all_pieces = 10;
+        m_bonus_one_piece = 0;
+    }
     else
     {
         m_bonus_all_pieces = 15;
@@ -293,12 +330,38 @@ void Board::init_variant(Variant variant)
     m_starting_points.init(variant, *m_geo);
     for (Color c : get_colors())
     {
-        if (variant == Variant::classic_2
-            || variant == Variant::trigon_2)
+        if (m_nu_players == 2 && m_nu_colors == 4)
             m_second_color[c] = get_next(get_next(c));
         else
             m_second_color[c] = c;
     }
+    auto piece_set = m_bc->get_piece_set();
+#if LIBBOARDGAME_DEBUG
+    unsigned total_piece_points = 0;
+#endif
+    for (Piece::IntType i = 0; i < get_nu_uniq_pieces(); ++i)
+    {
+        Piece piece(i);
+        m_score_points[piece] = get_piece_info(piece).get_score_points();
+#if LIBBOARDGAME_DEBUG
+        total_piece_points += m_score_points[piece];
+#endif
+    }
+    switch (piece_set)
+    {
+    case PieceSet::classic:
+        LIBBOARDGAME_ASSERT(total_piece_points == 89);
+        break;
+    case PieceSet::junior:
+        LIBBOARDGAME_ASSERT(total_piece_points == 44);
+        break;
+    case PieceSet::trigon:
+        LIBBOARDGAME_ASSERT(total_piece_points == 110);
+        break;
+    case PieceSet::nexos:
+        LIBBOARDGAME_ASSERT(total_piece_points == 84);
+        break;
+    }
 }
 
 bool Board::is_game_over() const
@@ -309,6 +372,36 @@ bool Board::is_game_over() const
     return true;
 }
 
+bool Board::is_legal(Color c, Move mv) const
+{
+    auto& info = get_move_info(mv);
+    if (! is_piece_left(c, info.get_piece()))
+        return false;
+    bool has_attach_point = false;
+    auto i = info.begin();
+    auto end = info.end();
+    do
+    {
+        if (m_state_color[c].forbidden[*i])
+            return false;
+        if (is_attach_point(*i, c))
+            has_attach_point = true;
+    }
+    while (++i != end);
+    if (has_attach_point)
+        return true;
+    if (! is_first_piece(c))
+        return false;
+    i = info.begin();
+    do
+        if (is_colorless_starting_point(*i)
+            || (is_colored_starting_point(*i)
+                && get_starting_point_color(*i) == c))
+            return true;
+    while (++i != end);
+    return false;
+}
+
 /** Remove forbidden points from attach point lists.
     The attach point lists do not guarantee that they contain only
     non-forbidden attach points because that would be too expensive to
@@ -401,6 +494,7 @@ void Board::write(ostream& out, bool mark_last_move) const
     auto board_type = get_board_type();
     bool is_trigon = (board_type == BoardType::trigon
                       || board_type == BoardType::trigon_3);
+    bool is_nexos = (board_type == BoardType::nexos);
     for (unsigned y = 0; y < height; ++y)
     {
         if (height - y < 10)
@@ -410,6 +504,7 @@ void Board::write(ostream& out, bool mark_last_move) const
         {
             Point p = m_geo->get_point(x, y);
             bool is_offboard = p.is_null();
+            auto point_type = m_geo->get_point_type(x, y);
             if ((x > 0 || (is_trigon && x == 0 && m_geo->is_onboard(x + 1, y)))
                     && ! is_offboard)
             {
@@ -417,8 +512,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 // Trigon board, a slash or backslash is used instead of the
                 // space to indicate the orientation of the triangles. A
                 // less-than/greater-than character is used instead of the
-                // space to mark the last piece played (the mark is not placed
-                // within the piece or off-board).
+                // space to mark the last piece played.
                 if (! last_mv.is_null()
                         && get_move_info(last_mv.move).contains(p)
                         && (x == 0 || ! m_geo->is_onboard(x - 1, y)
@@ -442,7 +536,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 else if (is_trigon)
                 {
                     set_color(out, "\x1B[1;30;47m");
-                    out << (m_geo->get_point_type(x, y) == 1 ? '\\' : '/');
+                    out << (point_type == 1 ? '\\' : '/');
                 }
                 else
                 {
@@ -455,11 +549,11 @@ void Board::write(ostream& out, bool mark_last_move) const
                 if (is_trigon && x > 0 && m_geo->is_onboard(x - 1, y))
                 {
                     set_color(out, "\x1B[1;30;47m");
-                    out << (m_geo->get_point_type(x, y) == 1 ? '\\' : '/');
+                    out << (point_type == 1 ? '\\' : '/');
                 }
                 else
                 {
-                    set_color(out, "\x1B[0m");
+                    set_color(out, is_nexos ? "\x1B[1;30;47m" : "\x1B[0m");
                     out << "  ";
                 }
             }
@@ -468,7 +562,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 PointState s = get_point_state(p);
                 if (s.is_empty())
                 {
-                    if (is_colored_starting_point(p))
+                    if (is_colored_starting_point(p) && ! is_nexos)
                     {
                         Color c = get_starting_point_color(p);
                         set_color(out, m_color_esc_sequence[c]);
@@ -482,14 +576,26 @@ void Board::write(ostream& out, bool mark_last_move) const
                     else
                     {
                         set_color(out, "\x1B[1;30;47m");
-                        out << (is_trigon ? ' ' : '.');
+                        if (is_trigon)
+                            out << ' ';
+                        else if (is_nexos && point_type == 1)
+                            out << '-';
+                        else if (is_nexos && point_type == 2)
+                            out << '|';
+                        else if (is_nexos && point_type == 0)
+                            out << '+';
+                        else
+                            out << '.';
                     }
                 }
                 else
                 {
                     Color color = s.to_color();
                     set_color(out, m_color_esc_sequence[color]);
-                    out << m_color_char[color];
+                    if (is_nexos && m_geo->get_point_type(p) == 0)
+                        out << '*'; // Uncrossable junction
+                    else
+                        out << m_color_char[color];
                 }
             }
         }
