@@ -14,7 +14,6 @@
 #include "Geometry.h"
 #include "Grid.h"
 #include "MoveList.h"
-#include "MoveMarker.h"
 #include "PointList.h"
 #include "PointState.h"
 #include "Setup.h"
@@ -22,6 +21,8 @@
 
 namespace libpentobi_base {
 
+class MoveMarker;
+
 //-----------------------------------------------------------------------------
 
 /** Blokus board.
@@ -34,7 +35,7 @@ public:
     /** Maximum number of pieces per player in any game variant. */
     static const unsigned max_pieces = Setup::max_pieces;
 
-    typedef ArrayList<Piece, BoardConst::max_pieces> PiecesLeftList;
+    typedef ArrayList<Piece, Piece::max_pieces> PiecesLeftList;
 
     static const unsigned max_player_moves = max_pieces;
 
@@ -44,7 +45,7 @@ public:
     /** Use ANSI escape sequences for colored text output in operator>> */
     static bool color_output;
 
-    Board(Variant variant);
+    explicit Board(Variant variant);
 
     /** Not implemented to avoid unintended copies.
         Use copy_from() to copy a board state. */
@@ -131,9 +132,12 @@ public:
         of a unique piece per player. */
     unsigned get_nu_left_piece(Color c, Piece piece) const;
 
-    /** Get number of points by a color including the bonus. */
+    /** Get number of points of a color including the bonus. */
     unsigned get_points(Color c) const;
 
+    /** Get number of bonus points of a color. */
+    unsigned get_bonus(Color c) const;
+
     /** Is a point a potential attachment point for a color.
         Does not check if the point is forbidden. */
     bool is_attach_point(Point p, Color c) const;
@@ -159,17 +163,17 @@ public:
     void copy_from(const Board& bd);
 
     /** Play a move.
-        @pre mv.is_regular()
+        @pre ! mv.is_null()
         @pre get_nu_moves() < max_game_moves */
     void play(Color c, Move mv);
 
     /** Play a move for the current color to play.
-        @pre mv.is_regular()
+        @pre ! mv.is_null()
         @pre get_nu_moves() < max_game_moves */
     void play(Move mv);
 
     /** Play a move.
-        @pre mv.move.is_regular()
+        @pre ! mv.move.is_null()
         @pre get_nu_moves() < max_game_moves */
     void play(ColorMove mv);
 
@@ -187,6 +191,9 @@ public:
 
     bool has_setup() const;
 
+    /** Get the total number of moves played by all colors.
+        Does not include setup pieces.
+        @see get_nu_onboard_pieces() */
     unsigned get_nu_moves() const;
 
     /** Get the number of pieces on board.
@@ -215,19 +222,22 @@ public:
     bool is_game_over() const;
 
     /** Check if a move is legal.
-        @pre mv.is_regular() */
+        @pre ! mv.is_null() */
     bool is_legal(Color c, Move mv) const;
 
+    /** Check if a move is legal that is not a starting move.
+        @pre ! mv.is_null() */
+    bool is_legal_nonfirst(Color c, Move mv) const;
+
     /** Check if a move is legal for the current color to play.
-        @pre mv.is_regular() */
+        @pre ! mv.is_null() */
     bool is_legal(Move mv) const;
 
     /** Check that point is not already occupied or adjacent to own color.
-        @param c
-        @param p The point. Off-board points are allowed and return true. */
+        Point::null() is an allowed argument and returns false. */
     bool is_forbidden(Point p, Color c) const;
 
-    const Grid<bool>& is_forbidden(Color c) const;
+    const GridExt<bool>& is_forbidden(Color c) const;
 
     /** Check that no points of move are already occupied or adjacent to own
         color.
@@ -250,6 +260,22 @@ public:
         are more than two players). */
     int get_score(Color c) const;
 
+    /** Specialized version of get_score().
+        @pre get_nu_colors() == 2 */
+    int get_score_twocolor(Color c) const;
+
+    /** Specialized version of get_score().
+        @pre get_nu_players() == 4 && get_nu_colors() == 4 */
+    int get_score_multicolor(Color c) const;
+
+    /** Specialized version of get_score().
+        @pre get_nu_players() > 2 */
+    int get_score_multiplayer(Color c) const;
+
+    /** Specialized version of get_score().
+        @pre get_nu_players() == 2 */
+    int get_score_twoplayer(Color c) const;
+
     /** Get the place of a player in the game result.
         @param c The color of the player.
         @param[out] place The place of the player with that color. The place
@@ -267,7 +293,11 @@ public:
     /** See BoardConst::from_string() */
     Move from_string(const string& s) const;
 
-    bool find_move(const MovePoints& points, Move& move) const;
+    bool find_move(const MovePoints& points, Move& mv) const;
+
+    bool find_move(const MovePoints& points, Piece piece, Move& mv) const;
+
+    const Transform* find_transform(Move mv) const;
 
     const PieceInfo& get_piece_info(Piece piece) const;
 
@@ -321,7 +351,7 @@ private:
     /** Color-dependent part of the board state. */
     struct StateColor
     {
-        Grid<bool> forbidden;
+        GridExt<bool> forbidden;
 
         Grid<bool> is_attach_point;
 
@@ -362,6 +392,12 @@ private:
     /** Bonus for playing the 1-piece last. */
     unsigned m_bonus_one_piece;
 
+    /** Score points of a piece.
+        This is equal to the number of points of a piece apart from game
+        variant Nexos, in which the piece points contain some junction
+        points. */
+    PieceMap<unsigned> m_score_points;
+
     /** See get_nu_piece_instances() */
     uint_fast8_t m_nu_piece_instances;
 
@@ -399,6 +435,7 @@ private:
 
     StartingPoints m_starting_points;
 
+
     void gen_moves(Color c, Point p, unsigned adj_status, MoveMarker& marker,
                    MoveList& moves) const;
 
@@ -429,9 +466,15 @@ private:
 };
 
 
-inline bool Board::find_move(const MovePoints& points, Move& move) const
+inline bool Board::find_move(const MovePoints& points, Move& mv) const
 {
-    return m_bc->find_move(points, move);
+    return m_bc->find_move(points, mv);
+}
+
+inline bool Board::find_move(const MovePoints& points, Piece piece,
+                             Move& mv) const
+{
+    return m_bc->find_move(points, piece, mv);
 }
 
 inline Move Board::from_string(const string& s) const
@@ -441,20 +484,10 @@ inline Move Board::from_string(const string& s) const
 
 inline unsigned Board::get_adj_status(Point p, Color c) const
 {
-    auto& adj_status_list = m_bc->get_adj_status_list(p);
-    auto i = adj_status_list.begin();
-    auto end = adj_status_list.end();
-    LIBBOARDGAME_ASSERT(i != end);
-    unsigned val = 1;
-    unsigned result = is_forbidden(*i, c) ? val : 0;
+    auto i = m_bc->get_adj_status_list(p).begin();
+    unsigned result = is_forbidden(*i, c); // bool converted to integer is 1
     for (unsigned j = 1; j < PrecompMoves::adj_status_nu_adj; ++j)
-    {
-        if (++i == end)
-            break;
-        val <<= 1;
-        if (is_forbidden(*i, c))
-            result |= val;
-    }
+        result |= (is_forbidden(*(++i), c) << j);
     return result;
 }
 
@@ -626,35 +659,54 @@ inline Color Board::get_previous(Color c) const
 inline int Board::get_score(Color c) const
 {
     if (m_nu_colors == 2)
-    {
-        LIBBOARDGAME_ASSERT(m_nu_players == 2);
-        unsigned points0 = get_points(Color(0));
-        unsigned points1 = get_points(Color(1));
-        if (c == Color(0))
-            return points0 - points1;
-        else
-            return points1 - points0;
-    }
+        return get_score_twocolor(c);
     else if (m_nu_players == 2)
-    {
-        LIBBOARDGAME_ASSERT(m_nu_colors == 4);
-        unsigned points0 = get_points(Color(0)) + get_points(Color(2));
-        unsigned points1 = get_points(Color(1)) + get_points(Color(3));
-        if (c == Color(0) || c == Color(2))
-            return points0 - points1;
-        else
-            return points1 - points0;
-    }
+        return get_score_multicolor(c);
     else
-    {
-        int score = 0;
-        auto nu_players = static_cast<Color::IntType>(m_nu_players);
-        for (Color i : get_colors())
-            if (i != c)
-                score -= get_points(i);
-        score = get_points(c) + score / (static_cast<int>(nu_players) - 1);
-        return score;
-    }
+        return get_score_multiplayer(c);
+}
+
+inline int Board::get_score_twocolor(Color c) const
+{
+    LIBBOARDGAME_ASSERT(m_nu_colors == 2);
+    unsigned points0 = get_points(Color(0));
+    unsigned points1 = get_points(Color(1));
+    if (c == Color(0))
+        return points0 - points1;
+    else
+        return points1 - points0;
+}
+
+inline int Board::get_score_twoplayer(Color c) const
+{
+    LIBBOARDGAME_ASSERT(m_nu_players == 2);
+    if (m_nu_colors == 2)
+        return get_score_twocolor(c);
+    else
+        return get_score_multicolor(c);
+}
+
+inline int Board::get_score_multicolor(Color c) const
+{
+    LIBBOARDGAME_ASSERT(m_nu_players == 2 && m_nu_colors == 4);
+    unsigned points0 = get_points(Color(0)) + get_points(Color(2));
+    unsigned points1 = get_points(Color(1)) + get_points(Color(3));
+    if (c == Color(0) || c == Color(2))
+        return points0 - points1;
+    else
+        return points1 - points0;
+}
+
+inline int Board::get_score_multiplayer(Color c) const
+{
+    LIBBOARDGAME_ASSERT(m_nu_players > 2);
+    int score = 0;
+    auto nu_players = static_cast<Color::IntType>(m_nu_players);
+    for (Color i : get_colors())
+        if (i != c)
+            score -= get_points(i);
+    score = get_points(c) + score / (static_cast<int>(nu_players) - 1);
+    return score;
 }
 
 inline Color Board::get_second_color(Color c) const
@@ -723,7 +775,7 @@ inline bool Board::is_forbidden(Point p, Color c) const
     return m_state_color[c].forbidden[p];
 }
 
-inline const Grid<bool>& Board::is_forbidden(Color c) const
+inline const GridExt<bool>& Board::is_forbidden(Color c) const
 {
     return m_state_color[c].forbidden;
 }
@@ -745,7 +797,7 @@ inline bool Board::is_legal(Move mv) const
     return is_legal(m_state_base.to_play, mv);
 }
 
-inline bool Board::is_legal(Color c, Move mv) const
+inline bool Board::is_legal_nonfirst(Color c, Move mv) const
 {
     auto& info = get_move_info(mv);
     if (! is_piece_left(c, info.get_piece()))
@@ -761,18 +813,7 @@ inline bool Board::is_legal(Color c, Move mv) const
             has_attach_point = true;
     }
     while (++i != end);
-    if (has_attach_point)
-        return true;
-    if (! is_first_piece(c))
-        return false;
-    i = info.begin();
-    do
-        if (is_colorless_starting_point(*i)
-            || (is_colored_starting_point(*i)
-                && get_starting_point_color(*i) == c))
-            return true;
-    while (++i != end);
-    return false;
+    return has_attach_point;
 }
 
 inline bool Board::is_piece_left(Color c, Piece piece) const
@@ -806,19 +847,22 @@ inline void Board::place(Color c, Move mv)
     }
     ++m_state_base.nu_onboard_pieces_all;
     ++state_color.nu_onboard_pieces;
-    state_color.points += piece_size;
+    state_color.points += m_score_points[piece];
     auto i = info.begin();
     auto end = i + piece_size;
     do
     {
-        m_state_base.point_state[*i] = c;
+        m_state_base.point_state[*i] = PointState(c);
         for_each_color([&](Color c) {
             m_state_color[c].forbidden[*i] = true;
         });
     }
     while (++i != end);
-    i = info_ext.begin_attach();
-    end = info_ext.end_attach();
+    end = info_ext.end_adj();
+    for (i = info_ext.begin_adj(); i != end; ++i)
+        state_color.forbidden[*i] = true;
+    LIBBOARDGAME_ASSERT(i == info_ext.begin_attach());
+    end += info_ext.size_attach_points;
     auto& attach_points = m_attach_points[c];
     auto n = attach_points.size();
     do
@@ -830,11 +874,6 @@ inline void Board::place(Color c, Move mv)
         }
     while (++i != end);
     attach_points.resize(n);
-    LIBBOARDGAME_ASSERT(i == info_ext.begin_adj());
-    end += info_ext.size_adj_points;
-    do
-        state_color.forbidden[*i] = true;
-    while (++i != end);
 }
 
 inline void Board::play(Color c, Move mv)
@@ -862,7 +901,8 @@ inline void Board::restore_snapshot()
     m_state_base.to_play = m_snapshot.state_base.to_play;
     m_state_base.nu_onboard_pieces_all =
         m_snapshot.state_base.nu_onboard_pieces_all;
-    m_state_base.point_state.copy_from(m_snapshot.state_base.point_state, geo);
+    m_state_base.point_state.memcpy_from(m_snapshot.state_base.point_state,
+                                         geo);
     for (Color c : get_colors())
     {
         const auto& snapshot_state = m_snapshot.state_color[c];
