@@ -10,6 +10,7 @@
 
 #include "Engine.h"
 
+#include "MoveMarker.h"
 #include "libboardgame_sgf/TreeReader.h"
 #include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/Log.h"
@@ -21,7 +22,6 @@ using libboardgame_gtp::Failure;
 using libboardgame_sgf::InvalidPropertyValue;
 using libboardgame_sgf::TreeReader;
 using libboardgame_sgf::util::get_last_node;
-using libboardgame_util::log;
 using libboardgame_util::ArrayList;
 using libboardgame_util::RandomGenerator;
 
@@ -48,7 +48,7 @@ Engine::Engine(Variant variant)
 void Engine::board_changed()
 {
     if (m_show_board)
-        log(get_board());
+        LIBBOARDGAME_LOG(get_board());
 }
 
 void Engine::cmd_all_legal(const Arguments& args, Response& response)
@@ -77,7 +77,7 @@ void Engine::cmd_final_score(Response& response)
     }
     else
     {
-        int score = bd.get_score(Color(0));
+        int score = bd.get_score_twoplayer(Color(0));
         if (score > 0)
             response << "B+" << score;
         else if (score < 0)
@@ -306,14 +306,15 @@ Color Engine::get_color_arg(const Arguments& args, unsigned i) const
     string s = args.get_tolower(i);
     auto& bd = get_board();
     auto variant = bd.get_variant();
-    switch (variant)
+    if (get_nu_colors(variant) == 2)
+    {
+        if (s == "blue" || s == "black" || s == "b")
+            return Color(0);
+        if (s == "green" || s == "white" || s == "w")
+            return Color(1);
+    }
+    else
     {
-    case Variant::classic:
-    case Variant::classic_2:
-    case Variant::classic_3:
-    case Variant::trigon:
-    case Variant::trigon_2:
-    case Variant::trigon_3:
         if (s == "1" || s == "blue")
             return Color(0);
         if (s == "2" || s == "yellow")
@@ -322,14 +323,6 @@ Color Engine::get_color_arg(const Arguments& args, unsigned i) const
             return Color(2);
         if (s == "4" || s == "green")
             return Color(3);
-        break;
-    case Variant::duo:
-    case Variant::junior:
-        if (s == "blue" || s == "black" || s == "b")
-            return Color(0);
-        if (s == "green" || s == "white" || s == "w")
-            return Color(1);
-        break;
     }
     throw Failure("invalid color argument '" + s + "'");
 }
@@ -377,7 +370,7 @@ void Engine::set_player(PlayerBase& player)
 void Engine::set_show_board(bool enable)
 {
     if (enable && ! m_show_board)
-        log(get_board());
+        LIBBOARDGAME_LOG(get_board());
     m_show_board = enable;
 }
 
