@@ -13,7 +13,6 @@
 #include "BoardUpdater.h"
 #include "BoardUtil.h"
 #include "NodeUtil.h"
-#include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/StringUtil.h"
 
 namespace libpentobi_base {
@@ -67,8 +66,8 @@ ColorMove PentobiTree::get_move(const SgfNode& node) const
     MovePoints points;
     if (! libpentobi_base::node_util::get_move(node, m_variant, c, points))
         return ColorMove::null();
-    if ( points.size() == 0)
-        // Older (unreleased?) versiions of Pentobi used empty move values
+    if (points.size() == 0)
+        // Older (unreleased?) versions of Pentobi used empty move values
         // to encode pass moves in search tree dumps but we don't support
         // pass moves Board anymore.
         return ColorMove::null();
@@ -107,43 +106,31 @@ const SgfNode* PentobiTree::get_node_before_move_number(
 
 string PentobiTree::get_player_name(Color c) const
 {
+    string name;
     auto& root = get_root();
-    switch (m_variant)
+    if (get_nu_players(m_variant) == 2)
     {
-    case Variant::classic:
-    case Variant::classic_3:
-    case Variant::trigon:
-    case Variant::trigon_3:
-        if (c == Color(0))
-            return root.get_property("P1", "");
-        if (c == Color(1))
-            return root.get_property("P2", "");
-        if (c == Color(2))
-            return root.get_property("P3", "");
-        if (c == Color(3))
-            return root.get_property("P4", "");
-        break;
-    case Variant::classic_2:
-    case Variant::trigon_2:
         if (c == Color(0) || c == Color(2))
-            return root.get_property("PB", "");
-        if (c == Color(1) || c == Color(3))
-            return root.get_property("PW", "");
-        break;
-    case Variant::duo:
-    case Variant::junior:
-        if (c == Color(0))
-            return root.get_property("PB", "");
-        if (c == Color(1))
-            return root.get_property("PW", "");
-        break;
+            name = root.get_property("PB", "");
+        else if (c == Color(1) || c == Color(2))
+            name = root.get_property("PW", "");
     }
-    LIBBOARDGAME_ASSERT(false);
-    return "";
+    else
+    {
+        if (c == Color(0))
+            name = root.get_property("P1", "");
+        else if (c == Color(1))
+            name = root.get_property("P2", "");
+        else if (c == Color(2))
+            name = root.get_property("P3", "");
+        else if (c == Color(3))
+            name = root.get_property("P4", "");
+    }
+    return name;
 }
 
 Setup::PlacementList PentobiTree::get_setup_property(const SgfNode& node,
-                                                        const char* id) const
+                                                     const char* id) const
 {
     vector<string> values = node.get_multi_property(id);
     Setup::PlacementList result;
@@ -152,6 +139,15 @@ Setup::PlacementList PentobiTree::get_setup_property(const SgfNode& node,
     return result;
 }
 
+Variant PentobiTree::get_variant(const SgfNode& root)
+{
+    string game = root.get_property("GM");
+    Variant variant;
+    if (! parse_variant(game, variant))
+        throw InvalidPropertyValue("GM", game);
+    return variant;
+}
+
 bool PentobiTree::has_main_variation_moves() const
 {
     auto node = &get_root();
@@ -166,10 +162,7 @@ bool PentobiTree::has_main_variation_moves() const
 
 void PentobiTree::init(unique_ptr<SgfNode>& root)
 {
-    string game = root->get_property("GM");
-    Variant variant;
-    if (! parse_variant(game, variant))
-        throw InvalidPropertyValue("GM", game);
+    Variant variant = get_variant(*root);
     SgfTree::init(root);
     m_variant = variant;
     init_board_const(variant);
@@ -282,12 +275,15 @@ void PentobiTree::set_player(const SgfNode& node, Color c)
 void PentobiTree::set_player_name(Color c, const string& name)
 {
     auto& root = get_root();
-    switch (m_variant)
+    if (get_nu_players(m_variant) == 2)
+    {
+        if (c == Color(0) || c == Color(2))
+            set_property(root, "PB", name);
+        else if (c == Color(1) || c == Color(3))
+            set_property(root, "PW", name);
+    }
+    else
     {
-    case Variant::classic:
-    case Variant::classic_3:
-    case Variant::trigon:
-    case Variant::trigon_3:
         if (c == Color(0))
             set_property(root, "P1", name);
         else if (c == Color(1))
@@ -296,29 +292,7 @@ void PentobiTree::set_player_name(Color c, const string& name)
             set_property(root, "P3", name);
         else if (c == Color(3))
             set_property(root, "P4", name);
-        else
-            LIBBOARDGAME_ASSERT(false);
-        return;
-    case Variant::classic_2:
-    case Variant::trigon_2:
-        if (c == Color(0) || c == Color(2))
-            set_property(root, "PB", name);
-        else if (c == Color(1) || c == Color(3))
-            set_property(root, "PW", name);
-        else
-            LIBBOARDGAME_ASSERT(false);
-        return;
-    case Variant::duo:
-    case Variant::junior:
-        if (c == Color(0))
-            set_property(root, "PB", name);
-        else if (c == Color(1))
-            set_property(root, "PW", name);
-        else
-            LIBBOARDGAME_ASSERT(false);
-        return;
     }
-    LIBBOARDGAME_ASSERT(false);
 }
 
 void PentobiTree::set_result(const SgfNode& node, int score)
