@@ -8,9 +8,7 @@
 #define LIBPENTOBI_PIECE_TRANSFORMS_H
 
 #include <vector>
-#include "libboardgame_base/Geometry.h"
 #include "libboardgame_base/Transform.h"
-#include "libboardgame_util/Assert.h"
 
 namespace libpentobi_base {
 
@@ -57,7 +55,7 @@ template<class T>
 const Transform* PieceTransforms::find() const
 {
     for (auto t : m_all)
-        if (dynamic_cast<const T*>(t) != nullptr)
+        if (dynamic_cast<const T*>(t))
             return t;
     return nullptr;
 }
