@@ -19,10 +19,10 @@ using libboardgame_base::SpShtStrRep;
 //-----------------------------------------------------------------------------
 
 /** Point (coordinate of on-board field) for Blokus game variants.
-    Supports RectGeometry up to size 20 and TrigonGeometry up to edge size
-    9. */
+    Supports RectGeometry up to size 20, TrigonGeometry up to edge size 9,
+    and NexosGeometry up to size 13. */
 typedef
-libboardgame_base::Point<486, 35, 20, unsigned short, SpShtStrRep> Point;
+libboardgame_base::Point<486, 35, 25, unsigned short, SpShtStrRep> Point;
 
 //-----------------------------------------------------------------------------
 
