@@ -15,37 +15,20 @@ using namespace std;
 
 //-----------------------------------------------------------------------------
 
-/** State of an on-board point for games in which the point state can be
-    a color or empty */
+/** State of an on-board point, which can be a color or empty */
 class PointState
 {
 public:
     typedef Color::IntType IntType;
 
-    class Iterator
-    {
-        friend class PointState;
-
-    public:
-        Iterator();
-
-        explicit operator bool() const;
-
-        void operator++();
-
-        PointState operator*() const;
-
-    private:
-        IntType m_i;
-    };
-
     static const IntType range = Color::range + 1;
 
     static const IntType value_empty = range - 1;
 
+
     PointState();
 
-    PointState(Color c);
+    explicit PointState(Color c);
 
     explicit PointState(IntType i);
 
@@ -75,28 +58,6 @@ private:
     bool is_initialized() const;
 };
 
-typedef PointState::Iterator PointStateIterator;
-
-inline PointState::Iterator::Iterator()
-    : m_i(0)
-{
-}
-
-inline PointState::Iterator::operator bool() const
-{
-    return m_i < PointState::range;
-}
-
-inline void PointState::Iterator::operator++()
-{
-    ++m_i;
-}
-
-inline PointState PointState::Iterator::operator*() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return PointState(m_i);
-}
 
 inline PointState::PointState()
 {
