@@ -25,14 +25,6 @@ namespace libpentobi_base {
 class PrecompMoves
 {
 public:
-    /** The maximum sum of the sizes of all precomputed move lists in any
-        game variant. */
-#if PENTOBI_LOW_RESOURCES
-    static const unsigned max_move_lists_sum_length = 832444;
-#else
-    static const unsigned max_move_lists_sum_length = 1425934;
-#endif
-
     /** The number of neighbors used for computing the adjacent status.
         The adjacent status is a single number that encodes the forbidden
         status of the first adj_status_nu_adj neighbors (from the list
@@ -43,15 +35,22 @@ public:
         Therefore, the optimal value for speeding up the matching depends on
         the CPU cache size. */
 #if PENTOBI_LOW_RESOURCES
-    static const unsigned adj_status_nu_adj = 4;
-#else
     static const unsigned adj_status_nu_adj = 5;
+#else
+    static const unsigned adj_status_nu_adj = 6;
 #endif
 
+    /** The maximum sum of the sizes of all precomputed move lists in any
+        game variant. */
+    static const unsigned max_move_lists_sum_length =
+            adj_status_nu_adj == 4 ?
+                832444 : adj_status_nu_adj == 5 ? 1425934 : 2769060;
+    static_assert(adj_status_nu_adj >= 4 && adj_status_nu_adj <= 6, "");
+
+    /** The range of values for the adjacent status. */
     static const unsigned nu_adj_status = 1 << adj_status_nu_adj;
 
-    /** Begin/end range for lists with moves at a given point.
-        See get_moves(). */
+    /** Begin/end range for lists with moves at a given point. */
     struct Range
     {
         const Move* m_begin;
@@ -63,14 +62,12 @@ public:
         const Move* end() const { return m_end; }
     };
 
-    /** Clear storage of moves for all move lists during construction. */
-    void clear() { m_move_lists.clear(); }
-
-    /** Get number of moves in all lists. */
-    unsigned get_size() const { return m_move_lists.size(); }
-
-    /** Add move to list during construction. */
-    void push_move(Move mv) { m_move_lists.push_back(mv); }
+    /** Add a move to list during construction. */
+    void set_move(unsigned i, Move mv)
+    {
+        LIBBOARDGAME_ASSERT(i < max_move_lists_sum_length);
+        m_move_lists[i] = mv;
+    }
 
     /** Store beginning and end of a local move list duing construction. */
     void set_list_range(Point p, unsigned adj_status, Piece piece,
@@ -84,11 +81,16 @@ public:
     Range get_moves(Piece piece, Point p, unsigned adj_status = 0) const
     {
         auto& range = m_moves_range[p][adj_status][piece];
-        auto begin = move_lists_begin() + range.begin;
-        auto end = begin + range.size;
+        auto begin = move_lists_begin() + range.begin();
+        auto end = begin + range.size();
         return Range{begin, end};
     }
 
+    bool has_moves(Piece piece, Point p, unsigned adj_status) const
+    {
+        return ! m_moves_range[p][adj_status][piece].empty();
+    }
+
     /** Begin of storage for move lists.
         Only needed for special use cases like during an in-place construction
         of PrecompMoves for follow-up positions when we need to compare the
@@ -98,23 +100,30 @@ public:
     const Move* move_lists_begin() const { return &(*m_move_lists.begin()); }
 
 private:
-    struct CompressedRange
+    class CompressedRange
     {
-        unsigned begin : 24;
-
-        unsigned size : 8;
-
-        CompressedRange() { }
+    public:
+        CompressedRange() = default;
 
         CompressedRange(unsigned begin, unsigned size)
         {
             LIBBOARDGAME_ASSERT(begin < max_move_lists_sum_length);
             LIBBOARDGAME_ASSERT(begin + size <= max_move_lists_sum_length);
-            LIBBOARDGAME_ASSERT(begin < (1 << 24));
-            this->begin = begin & ((1 << 24) - 1);
+            static_assert(max_move_lists_sum_length < (1 << 24), "");
             LIBBOARDGAME_ASSERT(size < (1 << 8));
-            this->size = size & ((1 << 8) - 1);
+            m_val = size;
+            if (size != 0)
+                m_val |= (begin << 8);
         }
+
+        bool empty() const { return m_val == 0; }
+
+        unsigned begin() const { return m_val >> 8; }
+
+        unsigned size() const { return m_val & 0xff; }
+
+    private:
+        uint_least32_t m_val;
     };
 
     /** See m_move_lists. */
@@ -124,7 +133,7 @@ private:
         constrained by the forbidden status of adjacent points.
         All lists are stored in a single array; m_moves_range contains
         information about the actual begin/end indices. */
-    ArrayList<Move, max_move_lists_sum_length> m_move_lists;
+    array<Move, max_move_lists_sum_length> m_move_lists;
 };
 
 //-----------------------------------------------------------------------------
