@@ -24,24 +24,17 @@ struct Setup
 
     typedef ArrayList<Move,max_pieces> PlacementList;
 
-    Color to_play;
+    Color to_play = Color(0);
 
     ColorMap<PlacementList> placements;
 
-    Setup();
-
     void clear();
 };
 
-inline Setup::Setup()
-    : to_play(Color(0))
-{
-}
-
 inline void Setup::clear()
 {
     to_play = Color(0);
-    LIBPENTOBI_FOREACH_COLOR(c, placements[c].clear());
+    for_each_color([&](Color c) { placements[c].clear(); });
 }
 
 //-----------------------------------------------------------------------------
