@@ -26,7 +26,9 @@ enum class PieceSet
 
     junior,
 
-    trigon
+    trigon,
+
+    nexos
 };
 
 //-----------------------------------------------------------------------------
@@ -39,7 +41,9 @@ enum class BoardType
 
     trigon,
 
-    trigon_3
+    trigon_3,
+
+    nexos
 };
 
 //-----------------------------------------------------------------------------
@@ -61,7 +65,11 @@ enum class Variant
 
     trigon_2,
 
-    trigon_3
+    trigon_3,
+
+    nexos,
+
+    nexos_2
 };
 
 //-----------------------------------------------------------------------------
