@@ -25,7 +25,7 @@ using libpentobi_base::ColorMap;
 
 //-----------------------------------------------------------------------------
 
-class ComputerColorDialog
+class ComputerColorDialog final
     : public QDialog
 {
     Q_OBJECT
@@ -46,7 +46,7 @@ private:
 
     void createCheckBox(QLayout* layout, Color c);
 
-    QString getPlayerString(Variant variant, Color c);
+    QString getPlayerString(Color c);
 };
 
 //-----------------------------------------------------------------------------
