@@ -44,8 +44,6 @@ public:
 
     PlayerInt get_player() const override;
 
-    Float get_tie_value() const override;
-
     bool check_followup(ArrayList<Move, max_moves>& sequence) override;
 
     string get_info() const override;
@@ -70,7 +68,7 @@ public:
 
 
     bool search(Move& mv, const Board& bd, Color to_play, Float max_count,
-                Float min_simulations, double max_time,
+                size_t min_simulations, double max_time,
                 TimeSource& time_source);
 
     /** Get color to play at root node of the last search. */
