@@ -23,24 +23,31 @@ using libpentobi_base::Piece;
 
 namespace {
 
-void filter_min_size(const BoardConst& board_const, unsigned min_size,
+void filter_min_size(const BoardConst& bc, unsigned min_size,
                      PieceMap<bool>& is_piece_considered)
 {
-    for (Piece::IntType i = 0; i < board_const.get_nu_pieces(); ++i)
+    for (Piece::IntType i = 0; i < bc.get_nu_pieces(); ++i)
     {
         Piece piece(i);
-        auto& piece_info = board_const.get_piece_info(piece);
-        if (piece_info.get_size() < min_size)
+        auto& piece_info = bc.get_piece_info(piece);
+        if (piece_info.get_score_points() < min_size)
             is_piece_considered[piece] = false;
     }
 }
 
-void set_piece_considered(const BoardConst& board_const, const char* name,
+/** Check if an adjacent status is a possible follow-up status for another
+    one. */
+inline bool is_followup_adj_status(unsigned status_new, unsigned status_old)
+{
+    return (status_new & status_old) == status_old;
+}
+
+void set_piece_considered(const BoardConst& bc, const char* name,
                           PieceMap<bool>& is_piece_considered,
                           bool is_considered = true)
 {
     Piece piece;
-    bool found = board_const.get_piece_by_name(name, piece);
+    bool found = bc.get_piece_by_name(name, piece);
     LIBBOARDGAME_UNUSED_IF_NOT_DEBUG(found);
     LIBBOARDGAME_ASSERT(found);
     is_piece_considered[piece] = is_considered;
@@ -49,55 +56,73 @@ void set_piece_considered(const BoardConst& board_const, const char* name,
 void set_pieces_considered(const Board& bd, unsigned nu_moves,
                            PieceMap<bool>& is_piece_considered)
 {
-    auto& board_const = bd.get_board_const();
-    auto board_type = board_const.get_board_type();
+    auto& bc = bd.get_board_const();
     unsigned nu_colors = bd.get_nu_colors();
     is_piece_considered.fill(true);
-    if (board_type == BoardType::duo)
+    switch (bc.get_board_type())
     {
+    case BoardType::duo:
         if (nu_moves < 2 * nu_colors)
-            filter_min_size(board_const, 5, is_piece_considered);
+            filter_min_size(bc, 5, is_piece_considered);
         else if (nu_moves < 3 * nu_colors)
-            filter_min_size(board_const, 4, is_piece_considered);
+            filter_min_size(bc, 4, is_piece_considered);
         else if (nu_moves < 5 * nu_colors)
-            filter_min_size(board_const, 3, is_piece_considered);
-    }
-    else if (board_type == BoardType::classic)
-    {
+            filter_min_size(bc, 3, is_piece_considered);
+        break;
+    case BoardType::classic:
         if (nu_moves < nu_colors)
         {
             is_piece_considered.fill(false);
-            set_piece_considered(board_const, "V5", is_piece_considered);
-            set_piece_considered(board_const, "Z5", is_piece_considered);
+            set_piece_considered(bc, "V5", is_piece_considered);
+            set_piece_considered(bc, "Z5", is_piece_considered);
+        }
+        else if (nu_moves < 2 * nu_colors)
+        {
+            filter_min_size(bc, 5, is_piece_considered);
+            set_piece_considered(bc, "F", is_piece_considered, false);
+            set_piece_considered(bc, "P", is_piece_considered, false);
+            set_piece_considered(bc, "T5", is_piece_considered, false);
+            set_piece_considered(bc, "U", is_piece_considered, false);
+            set_piece_considered(bc, "X", is_piece_considered, false);
         }
         else if (nu_moves < 3 * nu_colors)
-            filter_min_size(board_const, 5, is_piece_considered);
+        {
+            filter_min_size(bc, 5, is_piece_considered);
+            set_piece_considered(bc, "P", is_piece_considered, false);
+            set_piece_considered(bc, "U", is_piece_considered, false);
+        }
         else if (nu_moves < 5 * nu_colors)
-            filter_min_size(board_const, 4, is_piece_considered);
+            filter_min_size(bc, 4, is_piece_considered);
         else if (nu_moves < 7 * nu_colors)
-            filter_min_size(board_const, 3, is_piece_considered);
-    }
-    else if (board_type == BoardType::trigon
-             || board_type == BoardType::trigon_3)
-    {
+            filter_min_size(bc, 3, is_piece_considered);
+        break;
+    case BoardType::trigon:
+    case BoardType::trigon_3:
         if (nu_moves < nu_colors)
         {
             is_piece_considered.fill(false);
-            set_piece_considered(board_const, "V", is_piece_considered);
-            set_piece_considered(board_const, "I6", is_piece_considered);
+            set_piece_considered(bc, "V", is_piece_considered);
+            set_piece_considered(bc, "I6", is_piece_considered);
         }
         if (nu_moves < 4 * nu_colors)
         {
-            filter_min_size(board_const, 6, is_piece_considered);
+            filter_min_size(bc, 6, is_piece_considered);
             // O is a bad early move, it neither extends, nor blocks well
-            set_piece_considered(board_const, "O", is_piece_considered, false);
+            set_piece_considered(bc, "O", is_piece_considered, false);
         }
         else if (nu_moves < 5 * nu_colors)
-            filter_min_size(board_const, 5, is_piece_considered);
+            filter_min_size(bc, 5, is_piece_considered);
         else if (nu_moves < 7 * nu_colors)
-            filter_min_size(board_const, 4, is_piece_considered);
+            filter_min_size(bc, 4, is_piece_considered);
         else if (nu_moves < 9 * nu_colors)
-            filter_min_size(board_const, 3, is_piece_considered);
+            filter_min_size(bc, 3, is_piece_considered);
+        break;
+    case BoardType::nexos:
+        if (nu_moves < 3 * nu_colors)
+            filter_min_size(bc, 4, is_piece_considered);
+        else if (nu_moves < 5 * nu_colors)
+            filter_min_size(bc, 3, is_piece_considered);
+        break;
     }
 }
 
@@ -116,38 +141,37 @@ void SharedConst::init(bool is_followup)
 {
     auto& bd = *board;
     auto& bc = bd.get_board_const();
+
+    // Initialize precomp_moves
     for (Color c : bd.get_colors())
     {
-        auto& is_forbidden_at_root = this->is_forbidden_at_root[c];
-        is_forbidden_at_root.set();
+        auto& precomp_moves = this->precomp_moves[c];
+        const auto& old_precomp_moves =
+            (is_followup ? precomp_moves : bc.get_precomp_moves());
+
+        m_is_forbidden.set();
         for (Point p : bd)
             if (! bd.is_forbidden(p, c))
             {
                 auto adj_status = bd.get_adj_status(p, c);
                 for (Piece piece : bd.get_pieces_left(c))
-                    for (Move mv : bd.get_moves(piece, p, adj_status))
-                    {
-                        if (! is_forbidden_at_root[mv])
-                            continue;
-                        if (! bd.is_forbidden(c, mv))
-                            is_forbidden_at_root.clear(mv);
-                    }
+                    if (old_precomp_moves.has_moves(piece, p, adj_status))
+                        for (Move mv :
+                             old_precomp_moves.get_moves(piece, p, adj_status))
+                            if (m_is_forbidden[mv] && ! bd.is_forbidden(c, mv))
+                                m_is_forbidden.clear(mv);
             }
-    }
 
-    // Initialize m_shared_const.precomp_moves
-    for (Color c : bd.get_colors())
-    {
-        auto& precomp_moves = this->precomp_moves[c];
-        precomp_moves.clear();
-        // Construct new lists in-place from old if it is a follow-up position
-        const auto& old_precomp_moves =
-            (is_followup ? precomp_moves : bc.get_precomp_moves());
+        unsigned n = 0;
         for (Point p : bd)
         {
             if (bd.is_forbidden(p, c))
                 continue;
+            auto adj_status = bd.get_adj_status(p, c);
             for (unsigned i = 0; i < PrecompMoves::nu_adj_status; ++i)
+            {
+                if (! is_followup_adj_status(i, adj_status))
+                    continue;
                 // Don't iterate over bd.get_pieces_left(*i) because its
                 // ordering is not preserved if a piece is removed and the
                 // in-place construction requires that the iteration in these
@@ -159,28 +183,31 @@ void SharedConst::init(bool is_followup)
                     Piece piece(j);
                     if (! bd.is_piece_left(c, piece))
                         continue;
-                    auto begin = precomp_moves.get_size();
-                    auto moves = old_precomp_moves.get_moves(piece, p, i);
-                    for (auto& mv : moves)
-                        if (! is_forbidden_at_root[c][mv])
-                        {
-                            if (is_followup)
-                                // Assert that we don't overwrite old content
-                                // we still need during in-place construction
-                                LIBBOARDGAME_ASSERT(
-                                    &mv - precomp_moves.move_lists_begin()
-                                            >= static_cast<ptrdiff_t>(
-                                                precomp_moves.get_size()));
-                            precomp_moves.push_move(mv);
-                        }
-                    auto size = precomp_moves.get_size() - begin;
-                    precomp_moves.set_list_range(p, i, piece, begin, size);
+                    auto begin = n;
+                    if (old_precomp_moves.has_moves(piece, p, i))
+                        for (auto& mv :
+                             old_precomp_moves.get_moves(piece, p, i))
+                            if (! m_is_forbidden[mv])
+                                precomp_moves.set_move(n++, mv);
+                    precomp_moves.set_list_range(p, i, piece, begin,
+                                                 n - begin);
                 }
+            }
         }
     }
 
+    if (! is_followup)
+        init_pieces_considered();
+
+    symmetric_points.init(bd.get_geometry(), PointTransfRot180<Point>());
+}
+
+void SharedConst::init_pieces_considered()
+{
+    auto& bd = *board;
+    auto& bc = bd.get_board_const();
     is_piece_considered_list.clear();
-    for (unsigned i = 0; i < Board::max_game_moves; ++i)
+    for (auto i = bd.get_nu_onboard_pieces(); i < Board::max_game_moves; ++i)
     {
         PieceMap<bool> is_piece_considered;
         set_pieces_considered(bd, i, is_piece_considered);
@@ -208,8 +235,6 @@ void SharedConst::init(bool is_followup)
         }
     }
     is_piece_considered_all.fill(true);
-
-    symmetric_points.init(bd.get_geometry(), PointTransfRot180<Point>());
 }
 
 //-----------------------------------------------------------------------------
