@@ -11,8 +11,10 @@
 #include "State.h"
 
 #include "libboardgame_util/MathUtil.h"
-#include "libpentobi_base/BoardUtil.h"
 #include "libpentobi_base/ScoreUtil.h"
+#if LIBBOARDGAME_DEBUG
+#include "libpentobi_base/BoardUtil.h"
+#endif
 
 namespace libpentobi_mcts {
 
@@ -25,6 +27,13 @@ using libpentobi_base::PointState;
 
 namespace {
 
+/** Gamma value for PlayoutFeatures::get_nu_local().
+    The value of nu_local dominates all other features, so we use a high
+    gamma. Above some limit, we don't care about the exact value. */
+float gamma_local[PlayoutFeatures::max_local + 1] =
+  { 1, 1e6f, 1e12f, 1e18f, 1e24f, 1e30f, 1e30f, 1e30f, 1e30f, 1e30f, 1e30f,
+    1e30f, 1e30f, 1e30f, 1e30f };
+
 inline Float sigmoid(Float steepness, Float x)
 {
     return -1.f + 2.f / (1.f + fast_exp(-steepness * x));
@@ -40,44 +49,32 @@ State::State(Variant initial_variant, const SharedConst& shared_const)
 {
 }
 
+template<unsigned PIECE_SIZE>
 inline void State::add_moves(Point p, Color c,
-                             const Board::PiecesLeftList& pieces_considered)
+                             const Board::PiecesLeftList& pieces,
+                             float& total_gamma, MoveList& moves,
+                             unsigned& nu_moves)
 {
-    auto& moves = m_moves[c];
-    auto nu_moves = moves.size();
     auto& marker = m_marker[c];
     auto& playout_features = m_playout_features[c];
     auto adj_status = m_bd.get_adj_status(p, c);
-    double total_gamma =
-            moves.empty() ? 0 : m_cumulative_gamma[moves.size() - 1];
-    for (Piece piece : pieces_considered)
+    for (Piece piece : pieces)
+    {
+        if (! has_moves(c, piece, p, adj_status))
+            continue;
+        auto gamma_piece = m_gamma_piece[piece];
         for (Move mv : get_moves(c, piece, p, adj_status))
             if (! marker[mv]
-                    && check_move(mv, get_move_info(mv), moves, nu_moves,
-                                  playout_features, total_gamma))
+                    && check_move<PIECE_SIZE>(
+                        mv, get_move_info(mv), gamma_piece, moves, nu_moves,
+                        playout_features, total_gamma))
                 marker.set(mv);
-    moves.resize(nu_moves);
-    m_moves_added_at[c][p] = true;
+    }
 }
 
-inline void State::add_moves(Point p, Color c, Piece piece,
-                             unsigned adj_status, double& total_gamma)
-{
-    auto& moves = m_moves[c];
-    auto nu_moves = moves.size();
-    auto& marker = m_marker[c];
-    auto& playout_features = m_playout_features[c];
-    for (Move mv : get_moves(c, piece, p, adj_status))
-        if (! marker[mv]
-                && check_move(mv, get_move_info(mv), moves, nu_moves,
-                              playout_features, total_gamma))
-            marker.set(mv);
-    moves.resize(nu_moves);
-}
-
-void State::add_starting_moves(Color c,
-                               const Board::PiecesLeftList& pieces_considered,
-                               bool with_gamma)
+template<unsigned PIECE_SIZE>
+void State::add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
+                               bool with_gamma, MoveList& moves)
 {
     // Using only one starting point (if game variant has more than one) not
     // only reduces the branching factor but is also necessary because
@@ -86,17 +83,15 @@ void State::add_starting_moves(Color c,
     Point p = find_best_starting_point(c);
     if (p.is_null())
         return;
-    auto& moves = m_moves[c];
-    LIBBOARDGAME_ASSERT(moves.empty());
     unsigned nu_moves = 0;
     auto& marker = m_marker[c];
     auto& is_forbidden = m_bd.is_forbidden(c);
-    double total_gamma = 0;
-    for (Piece piece : pieces_considered)
+    float total_gamma = 0;
+    for (Piece piece : pieces)
         for (Move mv : get_moves(c, piece, p, 0))
         {
             LIBBOARDGAME_ASSERT(! marker[mv]);
-            if (check_forbidden(is_forbidden, mv, moves, nu_moves))
+            if (check_forbidden<PIECE_SIZE>(is_forbidden, mv, moves, nu_moves))
             {
                 marker.set(mv);
                 if (with_gamma)
@@ -109,48 +104,41 @@ void State::add_starting_moves(Color c,
     moves.resize(nu_moves);
 }
 
-bool State::check_forbidden(const Grid<bool>& is_forbidden, Move mv,
+template<unsigned PIECE_SIZE>
+bool State::check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
                             MoveList& moves, unsigned& nu_moves)
 {
     auto& info = get_move_info(mv);
     auto p = info.begin();
-    if (is_forbidden[*p])
+    unsigned forbidden = is_forbidden[*p];
+    for (unsigned i = 1; i < PIECE_SIZE; ++i)
+        // Logically, forbidden is a bool and the next line should be
+        //   forbidden = forbidden || is_forbidden[*(++p)]
+        // But this generates branches, which are bad for performance in this
+        // tight loop (unrolled by the compiler). So we use a bitwise OR, which
+        // works because C++ guarantees that true/false converts to 1/0.
+        forbidden |= static_cast<unsigned>(is_forbidden[*(++p)]);
+    if (forbidden != 0)
         return false;
-    auto end = info.end();
-    for (unsigned i = 1; i < PieceInfo::max_size; ++i)
-    {
-        if (++p == end)
-            break;
-        if (is_forbidden[*p])
-            return false;
-    }
     LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
     moves.get_unchecked(nu_moves) = mv;
     ++nu_moves;
     return true;
 }
 
-bool State::check_move(Move mv, const MoveInfo& info, MoveList& moves,
-                       unsigned& nu_moves,
+template<unsigned PIECE_SIZE>
+bool State::check_move(Move mv, const MoveInfo& info, float gamma_piece,
+                       MoveList& moves, unsigned& nu_moves,
                        const PlayoutFeatures& playout_features,
-                       double& total_gamma)
+                       float& total_gamma)
 {
     auto p = info.begin();
     PlayoutFeatures::Compute features(*p, playout_features);
+    for (unsigned i = 1; i < PIECE_SIZE; ++i)
+        features.add(*(++p), playout_features);
     if (features.is_forbidden())
         return false;
-    auto end = info.end();
-    for (unsigned i = 1; i < PieceInfo::max_size; ++i)
-    {
-        if (++p == end)
-            break;
-        if (! features.add(*p, playout_features))
-            return false;
-    }
-    double gamma = m_gamma_piece[info.get_piece()];
-    auto nu_local = features.get_nu_local();
-    if (nu_local > 0)
-        gamma *= m_gamma_nu_attach[nu_local];
+    auto gamma = gamma_piece * gamma_local[features.get_nu_local()];
     total_gamma += gamma;
     m_cumulative_gamma[nu_moves] = total_gamma;
     LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
@@ -159,6 +147,17 @@ bool State::check_move(Move mv, const MoveInfo& info, MoveList& moves,
     return true;
 }
 
+template<unsigned PIECE_SIZE>
+inline bool State::check_move(Move mv, const MoveInfo& info, MoveList& moves,
+                              unsigned& nu_moves,
+                              const PlayoutFeatures& playout_features,
+                              float& total_gamma)
+{
+    return check_move<PIECE_SIZE>(
+                mv, info, m_gamma_piece[info.get_piece()], moves, nu_moves,
+                playout_features, total_gamma);
+}
+
 #if LIBBOARDGAME_DEBUG
 string State::dump() const
 {
@@ -177,12 +176,12 @@ void State::evaluate_duo(array<Float, 6>& result)
             && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
     {
         if (log_simulations)
-            log("Result: 0.5 (symmetry)");
+            LIBBOARDGAME_LOG("Result: 0.5 (symmetry)");
         result[0] = result[1] = 0.5;
         return;
     }
     Color c(0);
-    auto s = static_cast<Float>(m_bd.get_score(c));
+    auto s = static_cast<Float>(m_bd.get_score_twocolor(c));
     Float res;
     if (s > 0)
         res = 1;
@@ -191,10 +190,10 @@ void State::evaluate_duo(array<Float, 6>& result)
     else
         res = 0.5;
     if (log_simulations)
-        log("Result color ", c, ": sco=", s, " game_res=", res);
+        LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=", res);
     res += get_quality_bonus(c, res, s, false);
     if (log_simulations)
-        log("res=", res);
+        LIBBOARDGAME_LOG("res=", res);
     result[0] = res;
     result[1] = 1.f - res;
 }
@@ -213,13 +212,13 @@ void State::evaluate_multicolor(array<Float, 6>& result)
             && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
     {
         if (log_simulations)
-            log("Result: 0.5 (symmetry)");
+            LIBBOARDGAME_LOG("Result: 0.5 (symmetry)");
         result[0] = result[1] = result[2] = result[3] = 0.5;
         return;
     }
 
     Color c(0);
-    auto s = static_cast<Float>(m_bd.get_score(c));
+    auto s = static_cast<Float>(m_bd.get_score_multicolor(c));
     Float res;
     if (s > 0)
         res = 1;
@@ -228,10 +227,10 @@ void State::evaluate_multicolor(array<Float, 6>& result)
     else
         res = 0.5;
     if (log_simulations)
-        log("Result color ", c, ": sco=", s, " game_res=", res);
+        LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=", res);
     res += get_quality_bonus(c, res, s, true);
     if (log_simulations)
-        log("res=", res);
+        LIBBOARDGAME_LOG("res=", res);
     result[0] = result[2] = res;
     result[1] = result[3] = 1.f - res;
 }
@@ -257,12 +256,12 @@ void State::evaluate_multiplayer(array<Float, 6>& result)
     for (Color::IntType i = 0; i < nu_players; ++i)
     {
         Color c(i);
-        auto s = static_cast<Float>(m_bd.get_score(c));
+        auto s = static_cast<Float>(m_bd.get_score_multiplayer(c));
         result[i] =
                 game_result[i] + get_quality_bonus(c, game_result[i], s, true);
         if (log_simulations)
-            log("Result color ", c, ": sco=", s, " game_res=", game_result[i],
-                " res=", result[i]);
+            LIBBOARDGAME_LOG("Result color ", c, ": sco=", s, " game_res=",
+                             game_result[i], " res=", result[i]);
     }
     if (m_bd.get_variant() == Variant::classic_3)
     {
@@ -276,17 +275,29 @@ Point State::find_best_starting_point(Color c) const
 {
     // We use the starting point that maximizes the distance to occupied
     // starting points, especially to the ones occupied by the player (their
-    // distance is weighted with a factor of 2)
+    // distance is weighted with a factor of 2).
     Point best = Point::null();
     float max_distance = -1;
-    bool is_trigon = (m_bd.get_board_type() == BoardType::trigon
-                      || m_bd.get_board_type() == BoardType::trigon_3);
+    auto board_type = m_bd.get_board_type();
+    bool is_trigon = (board_type == BoardType::trigon
+                      || board_type == BoardType::trigon_3);
+    bool is_nexos = board_type == BoardType::nexos;
     float ratio = (is_trigon ? 1.732f : 1);
     auto& geo = m_bd.get_geometry();
     for (Point p : m_bd.get_starting_points(c))
     {
         if (m_bd.is_forbidden(p, c))
             continue;
+        if (is_nexos)
+        {
+            // Don't use the starting segments towards the edge of the board
+            auto x = geo.get_x(p);
+            if (x <= 3 || x >= geo.get_width() - 3 - 1)
+                continue;
+            auto y = geo.get_y(p);
+            if (y <= 3 || y >= geo.get_height() - 3 - 1)
+                continue;
+        }
         float px = static_cast<float>(geo.get_x(p));
         float py = static_cast<float>(geo.get_y(p));
         float d = 0;
@@ -315,24 +326,75 @@ Point State::find_best_starting_point(Color c) const
     return best;
 }
 
+bool State::gen_children(Tree::NodeExpander& expander, Float init_val)
+{
+    if (m_nu_passes == m_nu_colors)
+        return true;
+    Color to_play = m_bd.get_to_play();
+    switch (m_piece_set)
+    {
+    case PieceSet::classic:
+    case PieceSet::junior:
+        init_moves_without_gamma<5>(to_play);
+        break;
+    case PieceSet::trigon:
+        init_moves_without_gamma<6>(to_play);
+        break;
+    case PieceSet::nexos:
+        init_moves_without_gamma<7>(to_play);
+        break;
+    }
+    return m_prior_knowledge.gen_children(m_bd, m_moves[to_play],
+                                          m_is_symmetry_broken, expander,
+                                          init_val);
+}
+
 bool State::gen_playout_move_full(PlayerMove<Move>& mv)
 {
     Color to_play = m_bd.get_to_play();
     while (true)
     {
         if (! m_is_move_list_initialized[to_play])
-            init_moves_with_gamma(to_play);
+        {
+            switch (m_piece_set)
+            {
+            case PieceSet::classic:
+            case PieceSet::junior:
+                init_moves_with_gamma<5>(to_play);
+                break;
+            case PieceSet::trigon:
+                init_moves_with_gamma<6>(to_play);
+                break;
+            case PieceSet::nexos:
+                init_moves_with_gamma<7>(to_play);
+                break;
+            }
+        }
         else if (m_has_moves[to_play])
-            update_moves(to_play);
+        {
+            switch (m_piece_set)
+            {
+            case PieceSet::classic:
+            case PieceSet::junior:
+                update_moves<5>(to_play);
+                break;
+            case PieceSet::trigon:
+                update_moves<6>(to_play);
+                break;
+            case PieceSet::nexos:
+                update_moves<7>(to_play);
+                break;
+            }
+        }
         if ((m_has_moves[to_play] = ! m_moves[to_play].empty()))
             break;
         if (++m_nu_passes == m_nu_colors)
             return false;
-        if (m_check_terminate_early && m_bd.get_score(to_play) < 0
+        if (m_check_terminate_early && m_bd.get_score_twoplayer(to_play) < 0
             && ! m_has_moves[m_bd.get_second_color(to_play)])
         {
             if (log_simulations)
-                log("Terminate early (no moves and negative score)");
+                LIBBOARDGAME_LOG("Terminate early (no moves and neg. score)");
             return false;
         }
         to_play = to_play.get_next(m_nu_colors);
@@ -342,12 +404,13 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
     }
 
     auto& moves = m_moves[to_play];
-    double total_gamma = m_cumulative_gamma[moves.size() - 1];
+    auto total_gamma = m_cumulative_gamma[moves.size() - 1];
     if (log_simulations)
-        log("Moves: ", moves.size(), ", total_gamma: ", total_gamma);
+        LIBBOARDGAME_LOG("Moves: ", moves.size(), ", total_gamma: ",
+                         total_gamma);
     auto begin = m_cumulative_gamma.begin();
     auto end = begin + moves.size();
-    auto random = m_random.generate_double(0, total_gamma);
+    auto random = m_random.generate_float(0, total_gamma);
     auto pos = lower_bound(begin, end, random);
     LIBBOARDGAME_ASSERT(pos != end);
     mv = PlayerMove<Move>(get_player(),
@@ -355,18 +418,46 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
     return true;
 }
 
-inline const PieceMap<bool>& State::get_pieces_considered() const
+string State::get_info() const
+{
+    ostringstream s;
+    if (m_bd.get_nu_players() == 2)
+    {
+        s << "Sco: ";
+        m_stat_score[Color(0)].write(s, true, 1);
+    }
+    s << '\n';
+    return s.str();
+}
+
+inline const PieceMap<bool>& State::get_is_piece_considered() const
 {
     // Use number of on-board pieces for move number to handle the case where
     // there are more pieces on the board than moves (setup positions)
     unsigned nu_moves = m_bd.get_nu_onboard_pieces();
     if (nu_moves >= m_shared_const.min_move_all_considered
-        || m_force_consider_all_pieces)
+            || m_force_consider_all_pieces)
         return m_shared_const.is_piece_considered_all;
     else
         return *m_shared_const.is_piece_considered[nu_moves];
 }
 
+/** Initializes and returns m_pieces_considered if not all pieces are
+    considered, otherwise m_bd.get_pieces_left(c) is returned. */
+inline const Board::PiecesLeftList& State::get_pieces_considered(Color c)
+{
+    auto is_piece_considered = m_is_piece_considered[c];
+    auto& pieces_left = m_bd.get_pieces_left(c);
+    if (is_piece_considered == &m_shared_const.is_piece_considered_all)
+        return pieces_left;
+    unsigned n = 0;
+    for (Piece piece : pieces_left)
+        if ((*is_piece_considered)[piece])
+            m_pieces_considered.get_unchecked(n++) = piece;
+    m_pieces_considered.resize(n);
+    return m_pieces_considered;
+}
+
 /** Bonus added to the result for quality-based rewards.
     See also: Pepels et al.: Quality-based Rewards for Monte-Carlo Tree Search
     Simulations. ECAI 2014. */
@@ -378,18 +469,18 @@ inline Float State::get_quality_bonus(Color c, Float result, Float score,
     // Game length
     Float l = static_cast<Float>(m_bd.get_nu_moves());
     m_stat_len.add(l);
-    Float dev = m_stat_len.get_deviation();
-    if (dev > 0)
+    Float var = m_stat_len.get_variance();
+    if (var > 0)
         bonus +=
                 (result == 1 ? -0.06f : 0.06f)
-                * sigmoid(2.f, (l - m_stat_len.get_mean()) / dev);
+                * sigmoid(2.f, (l - m_stat_len.get_mean()) / sqrt(var));
 
     // Game score
     auto& stat = m_stat_score[c];
     stat.add(score);
-    dev = stat.get_deviation();
-    if (dev > 0)
-        bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / dev);
+    var = stat.get_variance();
+    if (var > 0)
+        bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / sqrt(var));
 
     // Number of non-forbidden attach points is another feature of a superior
     // final position. Not used in Duo/Junior, mainly helps in Trigon.
@@ -414,71 +505,33 @@ inline Float State::get_quality_bonus(Color c, Float result, Float score,
             attach -= opp_weight * n;
         }
     m_stat_attach.add(attach);
-    dev = m_stat_attach.get_deviation();
-    if (dev > 0)
-        bonus += 0.1f
-                * sigmoid(2.f, (attach - m_stat_attach.get_mean()) / dev);
+    var = m_stat_attach.get_variance();
+    if (var > 0)
+        bonus +=
+          0.1f * sigmoid(2.f, (attach - m_stat_attach.get_mean()) / sqrt(var));
     return bonus;
 }
 
+template<unsigned PIECE_SIZE>
 void State::init_moves_with_gamma(Color c)
 {
-    m_is_piece_considered[c] = &get_pieces_considered();
-    auto& is_piece_considered = *m_is_piece_considered[c];
+    m_is_piece_considered[c] = &get_is_piece_considered();
     m_playout_features[c].set_local(m_bd);
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
-    moves.clear();
-    Board::PiecesLeftList pieces_considered;
-    for (Piece piece : m_bd.get_pieces_left(c))
-        if ((*m_is_piece_considered[c])[piece])
-            pieces_considered.push_back(piece);
+    auto& pieces = get_pieces_considered(c);
     if (m_bd.is_first_piece(c))
-        add_starting_moves(c, pieces_considered, true);
-    else
-        for (Point p : m_bd.get_attach_points(c))
-            if (! m_bd.is_forbidden(p, c))
-                add_moves(p, c, pieces_considered);
-    m_is_move_list_initialized[c] = true;
-    m_nu_new_moves[c] = 0;
-    m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
-    if (moves.empty()
-            && &is_piece_considered != &m_shared_const.is_piece_considered_all)
-    {
-        m_force_consider_all_pieces = true;
-        init_moves_with_gamma(c);
-    }
-}
-
-void State::init_moves_without_gamma(Color c)
-{
-    m_is_piece_considered[c] = &get_pieces_considered();
-    auto& is_piece_considered = *m_is_piece_considered[c];
-    auto& marker = m_marker[c];
-    auto& moves = m_moves[c];
-    marker.clear(moves);
-    moves.clear();
-    Board::PiecesLeftList pieces_considered;
-    for (Piece piece : m_bd.get_pieces_left(c))
-        if (is_piece_considered[piece])
-            pieces_considered.push_back(piece);
-    auto& is_forbidden = m_bd.is_forbidden(c);
-    if (m_bd.is_first_piece(c))
-        add_starting_moves(c, pieces_considered, false);
+        add_starting_moves<PIECE_SIZE>(c, pieces, true, moves);
     else
     {
         unsigned nu_moves = 0;
+        float total_gamma = 0;
         for (Point p : m_bd.get_attach_points(c))
-            if (! is_forbidden[p])
+            if (! m_bd.is_forbidden(p, c))
             {
-                auto adj_status = m_bd.get_adj_status(p, c);
-                for (Piece piece : pieces_considered)
-                    for (Move mv : get_moves(c, piece, p, adj_status))
-                        if (! marker[mv]
-                                && check_forbidden(is_forbidden, mv, moves,
-                                                   nu_moves))
-                            marker.set(mv);
+                add_moves<PIECE_SIZE>(
+                            p, c, pieces, total_gamma, moves, nu_moves);
                 m_moves_added_at[c][p] = true;
             }
         moves.resize(nu_moves);
@@ -486,18 +539,59 @@ void State::init_moves_without_gamma(Color c)
     m_is_move_list_initialized[c] = true;
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
-    if (moves.empty()
-            && &is_piece_considered != &m_shared_const.is_piece_considered_all)
+    if (moves.empty() && &pieces == &m_pieces_considered)
     {
         m_force_consider_all_pieces = true;
-        init_moves_without_gamma(c);
+        init_moves_with_gamma<PIECE_SIZE>(c);
+    }
+}
+
+template<unsigned PIECE_SIZE>
+void State::init_moves_without_gamma(Color c)
+{
+    m_is_piece_considered[c] = &get_is_piece_considered();
+    auto& marker = m_marker[c];
+    auto& moves = m_moves[c];
+    marker.clear(moves);
+    auto& pieces = get_pieces_considered(c);
+    auto& is_forbidden = m_bd.is_forbidden(c);
+    if (m_bd.is_first_piece(c))
+        add_starting_moves<PIECE_SIZE>(c, pieces, false, moves);
+    else
+    {
+        unsigned nu_moves = 0;
+        for (Point p : m_bd.get_attach_points(c))
+            if (! is_forbidden[p])
+            {
+                auto adj_status = m_bd.get_adj_status(p, c);
+                for (Piece piece : pieces)
+                {
+                    if (! has_moves(c, piece, p, adj_status))
+                        continue;
+                    for (Move mv : get_moves(c, piece, p, adj_status))
+                        if (! marker[mv]
+                                && check_forbidden<PIECE_SIZE>(
+                                    is_forbidden, mv, moves, nu_moves))
+                            marker.set(mv);
+                }
+                m_moves_added_at[c][p] = true;
+            }
+        moves.resize(nu_moves);
+    }
+    m_is_move_list_initialized[c] = true;
+    m_nu_new_moves[c] = 0;
+    m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
+    if (moves.empty() && &pieces == &m_pieces_considered)
+    {
+        m_force_consider_all_pieces = true;
+        init_moves_without_gamma<PIECE_SIZE>(c);
     }
 }
 
 void State::play_expanded_child(Move mv)
 {
     if (log_simulations)
-        log("Playing expanded child");
+        LIBBOARDGAME_LOG("Playing expanded child");
     if (! mv.is_null())
         play_playout(mv);
     else
@@ -510,7 +604,7 @@ void State::play_expanded_child(Move mv)
         // initialization)
         m_is_symmetry_broken = true;
         if (log_simulations)
-            log(m_bd);
+            LIBBOARDGAME_LOG(m_bd);
     }
 }
 
@@ -524,6 +618,7 @@ void State::start_search()
     for (Color c : Color::Range(m_nu_colors))
         m_playout_features[c].init_snapshot(m_bd, c);
     m_bc = &m_bd.get_board_const();
+    m_piece_set = m_bc->get_piece_set();
     m_move_info_array = m_bc->get_move_info_array();
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
     m_check_terminate_early =
@@ -553,8 +648,8 @@ void State::start_search()
         m_stat_score[c].clear();
 
     // Init gamma values
-    double gamma_size_factor = 1;
-    double gamma_nu_attach_factor = 1;
+    float gamma_size_factor = 1;
+    float gamma_nu_attach_factor = 1;
     switch (bd.get_board_type())
     {
     case BoardType::classic:
@@ -562,34 +657,42 @@ void State::start_search()
         break;
     case BoardType::duo:
         gamma_size_factor = 3;
-        gamma_nu_attach_factor = 1.8;
+        gamma_nu_attach_factor = 1.8f;
         break;
     case BoardType::trigon:
     case BoardType::trigon_3:
         gamma_size_factor = 5;
         break;
+    case BoardType::nexos:
+        // Not yet tuned
+        gamma_size_factor = 5;
+        gamma_nu_attach_factor = 1.8f;
+        break;
     }
     for (Piece::IntType i = 0; i < m_bc->get_nu_pieces(); ++i)
     {
         Piece piece(i);
-        auto piece_size = m_bc->get_piece_info(piece).get_size();
+        auto piece_size = static_cast<float>(
+                    m_bc->get_piece_info(piece).get_score_points());
+        auto piece_nu_attach =
+                static_cast<float>(m_bc->get_nu_attach_points(piece));
         LIBBOARDGAME_ASSERT(piece_size > 0);
-        auto piece_nu_attach = m_bc->get_nu_attach_points(piece);
         LIBBOARDGAME_ASSERT(piece_nu_attach > 0);
         m_gamma_piece[piece] =
             pow(gamma_size_factor, piece_size - 1)
             * pow(gamma_nu_attach_factor, piece_nu_attach - 1);
     }
-    for (unsigned i = 0; i < PieceInfo::max_size + 1; ++i)
-        m_gamma_nu_attach[i] = pow(1e10, i);
 }
 
 void State::start_simulation(size_t n)
 {
+#if LIBBOARDGAME_DISABLE_LOG
+    LIBBOARDGAME_UNUSED(n);
+#endif
     if (log_simulations)
-        log("=========================================================\n",
-            "Simulation ", n, "\n",
-            "=========================================================");
+        LIBBOARDGAME_LOG("=================================================\n",
+                         "Simulation ", n, "\n",
+                         "=================================================");
     m_bd.restore_snapshot();
     m_force_consider_all_pieces = false;
     auto& geo = m_bd.get_geometry();
@@ -603,6 +706,7 @@ void State::start_simulation(size_t n)
     m_nu_passes = 0;
 }
 
+template<unsigned PIECE_SIZE>
 void State::update_moves(Color c)
 {
     auto& playout_features = m_playout_features[c];
@@ -614,20 +718,19 @@ void State::update_moves(Color c)
     auto& is_forbidden = m_bd.is_forbidden(c);
     auto& moves = m_moves[c];
     auto old_size = moves.size();
-    moves.clear();
     unsigned nu_moves = 0;
-    double total_gamma = 0;
+    float total_gamma = 0;
     if (m_nu_new_moves[c] == 1 && m_bd.get_nu_piece_instances() == 1)
     {
         Piece piece = get_move_info(m_last_move[c]).get_piece();
         for (unsigned i = 0; i < old_size; ++i)
         {
-            LIBBOARDGAME_ASSERT(i >= moves.size());
-            Move mv = moves.get_unchecked(i);
+            LIBBOARDGAME_ASSERT(i >= nu_moves);
+            Move mv = moves[i];
             auto& info = get_move_info(mv);
             if (info.get_piece() == piece
-                    || ! check_move(mv, info, moves, nu_moves,
-                                    playout_features, total_gamma))
+                    || ! check_move<PIECE_SIZE>(mv, info, moves, nu_moves,
+                                                playout_features, total_gamma))
                 marker.clear(mv);
         }
     }
@@ -638,57 +741,54 @@ void State::update_moves(Color c)
             is_piece_left[piece] = true;
         for (unsigned i = 0; i < old_size; ++i)
         {
-            LIBBOARDGAME_ASSERT(i >= moves.size());
-            Move mv = moves.get_unchecked(i);
+            LIBBOARDGAME_ASSERT(i >= nu_moves);
+            Move mv = moves[i];
             auto& info = get_move_info(mv);
             if (! is_piece_left[info.get_piece()]
-                    || ! check_move(mv, info, moves,  nu_moves,
-                                    playout_features, total_gamma))
+                    || ! check_move<PIECE_SIZE>(mv, info, moves,  nu_moves,
+                                                playout_features, total_gamma))
                 marker.clear(mv);
         }
     }
-    moves.resize(nu_moves);
 
     // Find new legal moves because of new pieces played by this color
-    Board::PiecesLeftList pieces_considered;
-    for (Piece piece : m_bd.get_pieces_left(c))
-        if ((*m_is_piece_considered[c])[piece])
-            pieces_considered.push_back(piece);
+    auto& pieces = get_pieces_considered(c);
     auto& attach_points = m_bd.get_attach_points(c);
     auto begin = m_last_attach_points_end[c];
     auto end = attach_points.end();
     for (auto i = begin; i != end; ++i)
         if (! is_forbidden[*i] && ! m_moves_added_at[c][*i])
-            add_moves(*i, c, pieces_considered);
+        {
+            m_moves_added_at[c][*i] = true;
+            add_moves<PIECE_SIZE>(*i, c, pieces, total_gamma, moves, nu_moves);
+        }
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = end;
 
     // Generate moves for pieces not considered in the last position
-    auto& is_piece_considered = *m_is_piece_considered[c];
-    if (&is_piece_considered != &m_shared_const.is_piece_considered_all)
+    if (&pieces == &m_pieces_considered)
     {
-        if (moves.empty())
+        auto& is_piece_considered = *m_is_piece_considered[c];
+        if (nu_moves == 0)
             m_force_consider_all_pieces = true;
-        auto& is_piece_considered_new = get_pieces_considered();
+        auto& is_piece_considered_new = get_is_piece_considered();
         if (&is_piece_considered !=  &is_piece_considered_new)
         {
-            pieces_considered.clear();
+            Board::PiecesLeftList new_pieces;
+            unsigned n = 0;
             for (Piece piece : m_bd.get_pieces_left(c))
                 if (! is_piece_considered[piece]
-                    && is_piece_considered_new[piece])
-                    pieces_considered.push_back(piece);
-            double total_gamma =
-                    moves.empty() ? 0 : m_cumulative_gamma[moves.size() - 1];
-            for (Point p : m_bd.get_attach_points(c))
+                        && is_piece_considered_new[piece])
+                    new_pieces.get_unchecked(n++) = piece;
+            new_pieces.resize(n);
+            for (Point p : attach_points)
                 if (! is_forbidden[p])
-                {
-                    auto adj_status = m_bd.get_adj_status(p, c);
-                    for (Piece piece : pieces_considered)
-                        add_moves(p, c, piece, adj_status, total_gamma);
-                }
+                    add_moves<PIECE_SIZE>(p, c, new_pieces, total_gamma, moves,
+                                          nu_moves);
             m_is_piece_considered[c] = &is_piece_considered_new;
         }
     }
+    moves.resize(nu_moves);
 }
 
 void State::update_symmetry_broken(Move mv)
@@ -731,18 +831,6 @@ void State::update_symmetry_broken(Move mv)
     }
 }
 
-string State::get_info() const
-{
-    ostringstream s;
-    if (m_bd.get_nu_players() == 2)
-    {
-        s << "Sco: ";
-        m_stat_score[Color(0)].write(s, true, 1);
-    }
-    s << '\n';
-    return s.str();
-}
-
 //-----------------------------------------------------------------------------
 
 } // namespace libpentobi_mcts
