@@ -22,7 +22,6 @@ namespace libpentobi_mcts {
 using libboardgame_mcts::LastGoodReply;
 using libboardgame_mcts::PlayerInt;
 using libboardgame_mcts::PlayerMove;
-using libboardgame_util::log;
 using libboardgame_util::RandomGenerator;
 using libboardgame_util::Statistics;
 using libpentobi_base::BoardConst;
@@ -30,6 +29,7 @@ using libpentobi_base::MoveInfo;
 using libpentobi_base::MoveInfoExt;
 using libpentobi_base::Piece;
 using libpentobi_base::PieceInfo;
+using libpentobi_base::PieceSet;
 using libpentobi_base::Variant;
 
 //-----------------------------------------------------------------------------
@@ -84,7 +84,7 @@ public:
 
     void start_simulation(size_t n);
 
-    void gen_children(Tree::NodeExpander& expander, Float init_val);
+    bool gen_children(Tree::NodeExpander& expander, Float init_val);
 
     void start_playout();
 
@@ -111,7 +111,7 @@ private:
     static const bool log_simulations = false;
 
     /** The cumulative gamma value of the moves in m_moves. */
-    array<double, Move::range> m_cumulative_gamma;
+    array<float, MoveList::max_size> m_cumulative_gamma;
 
     Color::IntType m_nu_passes;
 
@@ -123,6 +123,8 @@ private:
 
     Color::IntType m_nu_colors;
 
+    PieceSet m_piece_set;
+
     const MoveInfo* m_move_info_array;
 
     const MoveInfoExt* m_move_info_ext_array;
@@ -134,13 +136,14 @@ private:
 
     ColorMap<const PieceMap<bool>*> m_is_piece_considered;
 
+    /** The list of pieces considered in the current move if not all pieces
+        are considered. */
+    Board::PiecesLeftList m_pieces_considered;
+
     PriorKnowledge m_prior_knowledge;
 
     /** Gamma value for a piece. */
-    PieceMap<double> m_gamma_piece;
-
-    /** Gamma value for PlayoutFeatures::get_nu_attach(). */
-    array<double, PieceInfo::max_size + 1> m_gamma_nu_attach;
+    PieceMap<float> m_gamma_piece;
 
     /** Number of moves played by a color since the last update of its move
         list. */
@@ -179,7 +182,11 @@ private:
 
     bool m_is_symmetry_broken;
 
-    /** Enforce all pieces to be considered for the rest of the simulation. */
+    /** Enforce all pieces to be considered for the rest of the simulation.
+        This applies to all colors, because it is only used if no moves were
+        generated because not all pieces were considered and this case is so
+        rare that it is not worth the cost of setting such a flag for each
+        color individually. */
     bool m_force_consider_all_pieces;
 
     /** Minimum number of pieces on board to perform a symmetry check.
@@ -197,16 +204,15 @@ private:
         played pieces. */
     ColorMap<Grid<bool>> m_moves_added_at;
 
-    void add_moves(Point p, Color c,
-                   const Board::PiecesLeftList& pieces_considered);
 
-    void add_moves(Point p, Color c, Piece piece, unsigned adj_status,
-                   double& total_gamma);
+    template<unsigned PIECE_SIZE>
+    void add_moves(Point p, Color c, const Board::PiecesLeftList& pieces,
+                   float& total_gamma, MoveList& moves, unsigned& nu_moves);
 
+    template<unsigned PIECE_SIZE>
     LIBBOARDGAME_NOINLINE
-    void add_starting_moves(Color c,
-                            const Board::PiecesLeftList& pieces_considered,
-                            bool with_gamma);
+    void add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
+                            bool with_gamma, MoveList& moves);
 
     void evaluate_duo(array<Float, 6>& result);
 
@@ -228,22 +234,37 @@ private:
     PrecompMoves::Range get_moves(Color c, Piece piece, Point p,
                                   unsigned adj_status) const;
 
-    const PieceMap<bool>& get_pieces_considered() const;
+    bool has_moves(Color c, Piece piece, Point p, unsigned adj_status) const;
 
+    const PieceMap<bool>& get_is_piece_considered() const;
+
+    const Board::PiecesLeftList& get_pieces_considered(Color c);
+
+    template<unsigned PIECE_SIZE>
     void init_moves_with_gamma(Color c);
 
+    template<unsigned PIECE_SIZE>
     void init_moves_without_gamma(Color c);
 
-    bool check_forbidden(const Grid<bool>& is_forbidden, Move mv,
+    template<unsigned PIECE_SIZE>
+    bool check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
                          MoveList& moves, unsigned& nu_moves);
 
+    template<unsigned PIECE_SIZE>
+    bool check_move(Move mv, const MoveInfo& info, float gamma_piece,
+                    MoveList& moves, unsigned& nu_moves,
+                    const PlayoutFeatures& playout_features,
+                    float& total_gamma);
+
+    template<unsigned PIECE_SIZE>
     bool check_move(Move mv, const MoveInfo& info, MoveList& moves,
                     unsigned& nu_moves,
                     const PlayoutFeatures& playout_features,
-                    double& total_gamma);
+                    float& total_gamma);
 
     bool gen_playout_move_full(PlayerMove<Move>& mv);
 
+    template<unsigned PIECE_SIZE>
     void update_moves(Color c);
 
     void update_playout_features(Color c, Move mv);
@@ -268,22 +289,12 @@ inline void State::evaluate_playout(array<Float, 6>& result)
 inline void State::finish_in_tree()
 {
     if (log_simulations)
-        log("Finish in-tree");
+        LIBBOARDGAME_LOG("Finish in-tree");
     if (m_check_symmetric_draw)
         m_is_symmetry_broken =
             check_symmetry_broken(m_bd, m_shared_const.symmetric_points);
 }
 
-inline void State::gen_children(Tree::NodeExpander& expander, Float init_val)
-{
-    if (m_nu_passes == m_nu_colors)
-        return;
-    Color to_play = m_bd.get_to_play();
-    init_moves_without_gamma(to_play);
-    m_prior_knowledge.gen_children(m_bd, m_moves[to_play],
-                                   m_is_symmetry_broken, expander, init_val);
-}
-
 inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
                                     Move second_last, PlayerMove<Move>& mv)
 {
@@ -294,23 +305,25 @@ inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
     {
         // See also the comment in evaluate_playout()
         if (log_simulations)
-            log("Terminate playout. Symmetry not broken.");
+            LIBBOARDGAME_LOG("Terminate playout. Symmetry not broken.");
         return false;
     }
     PlayerInt player = get_player();
     Move lgr2 = lgr.get_lgr2(player, last, second_last);
-    if (! lgr2.is_null() && m_bd.is_legal(lgr2))
+    // We use Board::is_legal_nonfirst() because it is faster and smaller
+    // and the cases that the lgr move is the first move of a color is rare.
+    if (! lgr2.is_null() && m_bd.is_legal_nonfirst(m_bd.get_to_play(), lgr2))
     {
         if (log_simulations)
-            log("Playing last good reply 2");
+            LIBBOARDGAME_LOG("Playing last good reply 2");
         mv = PlayerMove<Move>(player, lgr2);
         return true;
     }
     Move lgr1 = lgr.get_lgr1(player, last);
-    if (! lgr1.is_null() && m_bd.is_legal(lgr1))
+    if (! lgr1.is_null() && m_bd.is_legal_nonfirst(m_bd.get_to_play(), lgr1))
     {
         if (log_simulations)
-            log("Playing last good reply 1");
+            LIBBOARDGAME_LOG("Playing last good reply 1");
         mv = PlayerMove<Move>(player, lgr1);
         return true;
     }
@@ -345,6 +358,12 @@ inline PlayerInt State::get_player() const
     return static_cast<PlayerInt>(player);
 }
 
+inline bool State::has_moves(Color c, Piece piece, Point p,
+                             unsigned adj_status) const
+{
+    return m_shared_const.precomp_moves[c].has_moves(piece, p, adj_status);
+}
+
 inline void State::play_in_tree(Move mv)
 {
     Color to_play = m_bd.get_to_play();
@@ -361,7 +380,7 @@ inline void State::play_in_tree(Move mv)
         ++m_nu_passes;
     }
     if (log_simulations)
-        log(m_bd);
+        LIBBOARDGAME_LOG(m_bd);
 }
 
 inline void State::play_playout(Move mv)
@@ -375,7 +394,7 @@ inline void State::play_playout(Move mv)
     if (! m_is_symmetry_broken)
         update_symmetry_broken(mv);
     if (log_simulations)
-        log(m_bd);
+        LIBBOARDGAME_LOG(m_bd);
     update_playout_features(to_play, mv);
 }
 
