@@ -19,7 +19,7 @@ using namespace std;
 
 //-----------------------------------------------------------------------------
 
-class AnalyzeGameWindow
+class AnalyzeGameWindow final
     : public QDialog
 {
     Q_OBJECT
@@ -27,7 +27,8 @@ class AnalyzeGameWindow
 public:
     AnalyzeGameWidget* analyzeGameWidget;
 
-    AnalyzeGameWindow(QWidget* parent);
+
+    explicit AnalyzeGameWindow(QWidget* parent);
 };
 
 //-----------------------------------------------------------------------------
