@@ -224,10 +224,6 @@
         <source>&amp;Duo</source>
         <translation>&amp;Duo</translation>
     </message>
-    <message>
-        <source>J&amp;unior</source>
-        <translation>J&amp;unior</translation>
-    </message>
     <message>
         <source>&amp;Good</source>
         <translation>&amp;Gut</translation>
@@ -430,10 +426,6 @@
         <source>&amp;Undo Move</source>
         <translation>Zug rück&amp;gängig</translation>
     </message>
-    <message>
-        <source>Trigo&amp;n (3 Players)</source>
-        <translation>Trigo&amp;n (3 Spieler)</translation>
-    </message>
     <message>
         <source>V&amp;ery Bad</source>
         <translation>Seh&amp;r schlecht</translation>
@@ -632,26 +624,10 @@
         <source>Start a rated game</source>
         <translation>Ein gewertetes Spiel beginnen</translation>
     </message>
-    <message>
-        <source>&amp;Classic (4 Players)</source>
-        <translation>&amp;Klassisch (4 Spieler)</translation>
-    </message>
     <message>
         <source>Classic (&amp;3 Players)</source>
         <translation>Klassisch (&amp;3 Spieler)</translation>
     </message>
-    <message>
-        <source>Classic (&amp;2 Players)</source>
-        <translation>Klassisch (&amp;2 Spieler)</translation>
-    </message>
-    <message>
-        <source>&amp;Trigon (4 Players)</source>
-        <translation>&amp;Trigon (4 Spieler)</translation>
-    </message>
-    <message>
-        <source>Tri&amp;gon (2 Players)</source>
-        <translation>Tri&amp;gon (2 Spieler)</translation>
-    </message>
     <message>
         <source>&amp;Game</source>
         <translation>&amp;Spiel</translation>
@@ -867,6 +843,50 @@
         <source>Make the computer play the current color</source>
         <translation>Den Computer die gegenwärtige Farbe spielen lassen</translation>
     </message>
+    <message>
+        <source>Classic (&amp;4 Players)</source>
+        <translation>Klassisch (&amp;4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Junior</source>
+        <translation>&amp;Junior</translation>
+    </message>
+    <message>
+        <source>Classic (&amp;2 Players)</source>
+        <translation>Klassisch (&amp;2 Spieler)</translation>
+    </message>
+    <message>
+        <source>Nexos (&amp;2 Players)</source>
+        <translation>Nexos (&amp;2 Spieler)</translation>
+    </message>
+    <message>
+        <source>Nexos (&amp;4 Players)</source>
+        <translation>Nexos (&amp;4 Spieler)</translation>
+    </message>
+    <message>
+        <source>Trigon (&amp;2 Players)</source>
+        <translation>Trigon (&amp;2 Spieler)</translation>
+    </message>
+    <message>
+        <source>Trigon (&amp;3 Players)</source>
+        <translation>Trigon (&amp;3 Spieler)</translation>
+    </message>
+    <message>
+        <source>Trigon (&amp;4 Players)</source>
+        <translation>Trigon (&amp;4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Classic</source>
+        <translation>&amp;Klassisch</translation>
+    </message>
+    <message>
+        <source>&amp;Trigon</source>
+        <translation>&amp;Trigon</translation>
+    </message>
+    <message>
+        <source>&amp;Nexos</source>
+        <translation>&amp;Nexos</translation>
+    </message>
 </context>
 <context>
     <name>RatedGamesList</name>
@@ -977,5 +997,24 @@
         <source>Recent development:</source>
         <translation>Aktuelle Entwicklung:</translation>
     </message>
+    <message>
+        <source>Nexos (4 players)</source>
+        <translation>Nexos (4 Spieler)</translation>
+    </message>
+    <message>
+        <source>Nexos (2 players)</source>
+        <translation>Nexos (2 Spieler)</translation>
+    </message>
+</context>
+<context>
+    <name>main</name>
+    <message>
+        <source>Not enough memory.</source>
+        <translation>Nicht genügend Speicher.</translation>
+    </message>
+    <message>
+        <source>Pentobi</source>
+        <translation>Pentobi</translation>
+    </message>
 </context>
 </TS>
