@@ -9,7 +9,6 @@
 #endif
 
 #include <fstream>
-#include <iostream>
 #include "Engine.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/Options.h"
@@ -17,14 +16,12 @@
 
 using namespace std;
 using libboardgame_gtp::Failure;
-using libboardgame_util::get_log;
-using libboardgame_util::log;
-using libboardgame_util::set_log_null;
 using libboardgame_util::Options;
 using libboardgame_util::RandomGenerator;
 using libpentobi_base::parse_variant_id;
 using libpentobi_base::Board;
 using libpentobi_base::Variant;
+using libpentobi_mcts::Player;
 
 //-----------------------------------------------------------------------------
 
@@ -62,7 +59,6 @@ int main(int argc, char** argv)
             "game|g:",
             "help|h",
             "level|l:",
-            "memory:",
             "nobook",
             "noresign",
             "quiet|q",
@@ -84,7 +80,6 @@ int main(int argc, char** argv)
                 "             duo, trigon, trigon_2, trigon_3, junior)\n"
                 "--help,-h    print help message and exit\n"
                 "--level,-l   set playing strength level\n"
-                "--memory     memory to allocate for search trees\n"
                 "--seed,-r    set random seed\n"
                 "--showboard  automatically write board to stderr after\n"
                 "             changes\n"
@@ -104,13 +99,6 @@ int main(int argc, char** argv)
 #endif
             return 0;
         }
-        size_t memory = 0;
-        if (opt.contains("memory"))
-        {
-            memory = opt.get<size_t>("memory");
-            if (memory == 0)
-                throw runtime_error("Value for memory must be greater zero.");
-        }
         unsigned threads = 1;
         if (opt.contains("threads"))
         {
@@ -120,20 +108,21 @@ int main(int argc, char** argv)
         }
         Board::color_output = opt.contains("color");
         if (opt.contains("quiet"))
-            set_log_null();
+            libboardgame_util::disable_logging();
         if (opt.contains("seed"))
-            RandomGenerator::set_global_seed(opt.get<uint32_t>("seed"));
+            RandomGenerator::set_global_seed(
+                        opt.get<RandomGenerator::ResultType>("seed"));
         string variant_string = opt.get("game", "classic");
         Variant variant;
         if (! parse_variant_id(variant_string, variant))
             throw runtime_error("invalid game variant " + variant_string);
-        auto level = opt.get<int>("level", 4);
-        if (level < 1 || level > 9)
-            throw runtime_error("invalid level (must be 1-9)");
+        auto level = opt.get<unsigned>("level", 4);
+        if (level < 1 || level > Player::max_supported_level)
+            throw runtime_error("invalid level");
         auto use_book = (! opt.contains("nobook"));
         string books_dir = application_dir_path;
         pentobi_gtp::Engine engine(variant, level, use_book, books_dir,
-                                   threads, memory);
+                                   threads);
         engine.set_resign(! opt.contains("noresign"));
         if (opt.contains("showboard"))
             engine.set_show_board(true);
@@ -153,7 +142,7 @@ int main(int argc, char** argv)
             ifstream in(config_file);
             if (! in)
                 throw runtime_error("Error opening " + config_file);
-            engine.exec(in, true, get_log());
+            engine.exec(in, true, libboardgame_util::get_log_stream());
         }
         auto& args = opt.get_args();
         if (! args.empty())
@@ -170,12 +159,12 @@ int main(int argc, char** argv)
     }
     catch (const Failure& e)
     {
-        log("Error: command in config file failed: ", e.what());
+        LIBBOARDGAME_LOG("Error: command in config file failed: ", e.what());
         return 1;
     }
     catch (const exception& e)
     {
-        log("Error: ", e.what());
+        LIBBOARDGAME_LOG("Error: ", e.what());
         return 1;
     }
 }
