@@ -25,6 +25,8 @@ bool PentobiThumbCreator::create(const QString& path, int width, int height,
                                  QImage& image)
 {
     image = QImage(width, height, QImage::Format_ARGB32);
+    if (image.isNull())
+        return false;
     image.fill(Qt::transparent);
     return createThumbnail(path, width, height, image);
 }
