@@ -1,89 +1,40 @@
-function autosave() {
-    if (boardModel.isGameOver)
-        boardModel.clearAutoSave()
-    else
-        boardModel.autoSave()
-}
-
 function cancelGenMove() {
     playerModel.cancelGenMove()
     gameDisplay.busyIndicatorRunning = false
 }
 
 function changeGameVariant(gameVariant, verifyAbortGame) {
-    if (boardModel.gameVariant === gameVariant)
+    if (gameModel.gameVariant === gameVariant)
         return
     cancelGenMove()
-    if (! boardModel.isBoardEmpty && ! boardModel.isGameOver &&
+    if (! gameModel.isGameEmpty && ! gameModel.isGameOver &&
             verifyAbortGame) {
-        showMessageDialog(qsTr("New game?"),
-                          function() {
-                              changeGameVariant(gameVariant, false) })
+        showQuestion(qsTr("New game?"),
+                     function() { changeGameVariant(gameVariant, false) })
         return
     }
     callDelayTimer.call(function() {
         initGameVariant(gameVariant)
-        gameDisplay.transitionsEnabled = false
         initComputerColors()
-        gameDisplay.showToPlay()
-        gameDisplay.transitionsEnabled = true
     })
 }
 
 function checkComputerMove() {
-    if (boardModel.isGameOver) {
+    if (gameModel.isGameOver) {
         showGameOver()
         return
     }
     if (! isComputerToPlay())
         return
-    switch (boardModel.toPlay) {
-    case 0: if (! boardModel.hasMoves0) return; break
-    case 1: if (! boardModel.hasMoves1) return; break
-    case 2: if (! boardModel.hasMoves2) return; break
-    case 3: if (! boardModel.hasMoves3) return; break
+    switch (gameModel.toPlay) {
+    case 0: if (! gameModel.hasMoves0) return; break
+    case 1: if (! gameModel.hasMoves1) return; break
+    case 2: if (! gameModel.hasMoves2) return; break
+    case 3: if (! gameModel.hasMoves3) return; break
     }
     genMove();
 }
 
-function clearMarks() {
-    if (_pieceMarked0 != null) {
-        _pieceMarked0.isMarked = false
-        _pieceMarked0 = null
-    }
-    if (_pieceMarked1 != null) {
-        _pieceMarked1.isMarked = false
-        _pieceMarked1 = null
-    }
-    if (_pieceMarked2 != null) {
-        _pieceMarked2.isMarked = false
-        _pieceMarked2 = null
-    }
-    if (_pieceMarked3 != null) {
-        _pieceMarked3.isMarked = false
-        _pieceMarked3 = null
-    }
-}
-
-function clearMarksNotComputer() {
-    if (! computerPlays0 && _pieceMarked0 != null) {
-        _pieceMarked0.isMarked = false
-        _pieceMarked0 = null
-    }
-    if (! computerPlays1 && _pieceMarked1 != null) {
-        _pieceMarked1.isMarked = false
-        _pieceMarked1 = null
-    }
-    if (! computerPlays2 && _pieceMarked2 != null) {
-        _pieceMarked2.isMarked = false
-        _pieceMarked2 = null
-    }
-    if (! computerPlays3 && _pieceMarked3 != null) {
-        _pieceMarked3.isMarked = false
-        _pieceMarked3 = null
-    }
-}
-
 /** If the computer already plays the current color to play, start generating
     a move; if he doesn't, make him play the current color (and only the
     current color). */
@@ -96,9 +47,9 @@ function computerPlay() {
         computerPlays1 = false
         computerPlays2 = false
         computerPlays3 = false
-        var variant = boardModel.gameVariant
-        if (variant == "classic_3" && boardModel.toPlay == 3) {
-            switch (boardModel.altPlayer) {
+        var variant = gameModel.gameVariant
+        if (variant == "classic_3" && gameModel.toPlay == 3) {
+            switch (gameModel.altPlayer) {
             case 0: computerPlays0 = true; break
             case 1: computerPlays1 = true; break
             case 2: computerPlays2 = true; break
@@ -108,7 +59,7 @@ function computerPlay() {
         {
             var isMultiColor =
                     (variant == "classic_2" || variant == "trigon_2")
-            switch (boardModel.toPlay) {
+            switch (gameModel.toPlay) {
             case 0:
                 computerPlays0 = true
                 if (isMultiColor) computerPlays2 = true
@@ -141,7 +92,7 @@ function computerPlays(color) {
 }
 
 function computerPlaysAll() {
-    switch (boardModel.gameVariant) {
+    switch (gameModel.gameVariant) {
     case "duo":
     case "junior":
     case "classic_2":
@@ -158,38 +109,42 @@ function computerPlaysAll() {
 function createTheme(themeName) {
     var source = "qrc:///qml/themes/" + themeName + "/Theme.qml"
     var component = Qt.createComponent(source)
-    if (component.status != Component.Ready)
+    if (component.status !== Component.Ready)
         throw "Could not load " + source
     return component.createObject(root)
 }
 
 function genMove() {
     cancelGenMove()
-    gameDisplay.pickedPiece = null
+    gameDisplay.dropPiece()
     gameDisplay.busyIndicatorRunning = true
-    gameDisplay.showToPlay()
     isMoveHintRunning = false
-    playerModel.startGenMove(boardModel)
+    playerModel.startGenMove(gameModel)
+}
+
+function getFileFromUrl(fileUrl) {
+    var file = fileUrl.toString()
+    file = file.replace(/^(file:\/{3})/,"/")
+    return decodeURIComponent(file)
 }
 
 function hideComputerColorDialog()
 {
-    if (computerColorDialogLoader.status == Loader.Ready)
+    if (computerColorDialogLoader.status === Loader.Ready)
         computerColorDialogLoader.item.visible = false
 }
 
 function init() {
     callDelayTimer.call(function() {
-        gameDisplay.createPieces()
-        if (! boardModel.loadAutoSave())
+        if (! gameModel.loadAutoSave()) {
+            gameDisplay.createPieces()
             initComputerColors()
+        }
         else {
-            clearMarks()
-            markLastMove()
+            gameDisplay.createPieces()
             if (! computerPlaysAll())
                 checkComputerMove()
         }
-        gameDisplay.transitionsEnabled = true
     })
 }
 
@@ -199,52 +154,24 @@ function initComputerColors() {
     computerPlays1 = true
     computerPlays2 = true
     computerPlays3 = true
-    if (boardModel.gameVariant == "classic_2"
-            || boardModel.gameVariant == "trigon_2")
+    if (gameModel.gameVariant == "classic_2"
+            || gameModel.gameVariant == "trigon_2")
         computerPlays2 = false
 }
 
 function initGameVariant(gameVariant) {
     cancelGenMove()
     hideComputerColorDialog()
-    gameDisplay.clearMessage()
-    clearMarks()
     gameDisplay.destroyPieces()
-    boardModel.initGameVariant(gameVariant)
+    gameModel.initGameVariant(gameVariant)
     gameDisplay.createPieces()
+    gameDisplay.showToPlay()
 }
 
 function isComputerToPlay() {
-    if (boardModel.gameVariant == "classic_3" && boardModel.toPlay == 3)
-        return computerPlays(boardModel.altPlayer)
-    return computerPlays(boardModel.toPlay)
-}
-
-function markLastMove() {
-    if (! root.markLastMove)
-        return
-    var piece = gameDisplay.findPiece(boardModel.getLastMovePieceModel())
-    if (piece == null)
-        return
-    switch (boardModel.getLastMoveColor()) {
-    case 0:
-        if (_pieceMarked0 != null) _pieceMarked0.isMarked = false
-        _pieceMarked0 = piece
-        break
-    case 1:
-        if (_pieceMarked1 != null) _pieceMarked1.isMarked = false
-        _pieceMarked1 = piece
-        break
-    case 2:
-        if (_pieceMarked2 != null) _pieceMarked2.isMarked = false
-        _pieceMarked2 = piece
-        break
-    case 3:
-        if (_pieceMarked3 != null) _pieceMarked3.isMarked = false
-        _pieceMarked3 = piece
-        break
-    }
-    piece.isMarked = true
+    if (gameModel.gameVariant == "classic_3" && gameModel.toPlay == 3)
+        return computerPlays(gameModel.altPlayer)
+    return computerPlays(gameModel.toPlay)
 }
 
 function moveGenerated(move) {
@@ -254,58 +181,81 @@ function moveGenerated(move) {
         isMoveHintRunning = false
         return
     }
-    boardModel.playMove(move)
-    if (computerPlaysAll())
-        clearMarks()
-    else
-        clearMarksNotComputer()
-    markLastMove()
+    gameDisplay.transitionsEnabled = true
+    gameModel.playMove(move)
+    gameDisplay.transitionsEnabled = false
     delayedCheckComputerMove.start()
 }
 
 function moveHint() {
-    if (boardModel.isGameOver)
+    if (gameModel.isGameOver)
         return
     cancelGenMove()
     isMoveHintRunning = true
     gameDisplay.busyIndicatorRunning = true
-    playerModel.startGenMoveAtLevel(boardModel, 1)
+    playerModel.startGenMoveAtLevel(gameModel, 1)
 }
 
 function newGame(verifyAbortGame)
 {
     cancelGenMove()
-    if (! boardModel.isBoardEmpty &&  ! boardModel.isGameOver &&
+    if (! gameModel.isGameEmpty &&  ! gameModel.isGameOver &&
             verifyAbortGame) {
-        showMessageDialog(qsTr("New game?"),
-                          function() { newGame(false) })
+        showQuestion(qsTr("New game?"), function() { newGame(false) })
         return
     }
-    gameDisplay.pickedPiece = null
-    gameDisplay.clearMessage()
-    clearMarks()
+    gameDisplay.dropPiece()
     hideComputerColorDialog()
-    gameDisplay.transitionsEnabled = false
-    boardModel.newGame()
+    gameModel.newGame()
     gameDisplay.showToPlay()
-    gameDisplay.transitionsEnabled = true
     initComputerColors()
 }
 
+function open() {
+    if (openDialogLoader.status === Loader.Null)
+        openDialogLoader.sourceComponent = openDialogComponent
+    openDialogLoader.item.open()
+}
+
+function openFileUrl(fileUrl) {
+    cancelGenMove()
+    gameDisplay.destroyPieces()
+    if (! gameModel.open(getFileFromUrl(fileUrl)))
+        showError(qsTr("Open failed.") + "\n" + gameModel.lastInputOutputError)
+    else {
+        computerPlays0 = false
+        computerPlays1 = false
+        computerPlays2 = false
+        computerPlays3 = false
+    }
+    gameDisplay.createPieces()
+    gameDisplay.showToPlay()
+}
+
 function play(pieceModel, gameCoord) {
     cancelGenMove()
     var wasComputerToPlay = isComputerToPlay()
-    boardModel.play(pieceModel, gameCoord)
-    clearMarks()
-    markLastMove()
-    // If a move was entered but the computer plays this color (e.g. after
-    // undo move) we don't automatically continue playing.
+    gameDisplay.transitionsEnabled = true
+    gameModel.playPiece(pieceModel, gameCoord)
+    // We don't continue automatic play if the human played a move for a color
+    // played by the computer.
     if (! wasComputerToPlay)
         delayedCheckComputerMove.start()
 }
 
+function saveFileUrl(fileUrl) {
+    if (! gameModel.save(getFileFromUrl(fileUrl)))
+        showError(qsTr("Save failed.") + "\n" + gameModel.lastInputOutputError)
+}
+
+function saveAs() {
+    if (saveDialogLoader.status === Loader.Null)
+        saveDialogLoader.sourceComponent = saveDialogComponent
+    saveDialogLoader.item.open()
+}
+
 function showComputerColorDialog() {
-    if (computerColorDialogLoader.status == Loader.Null)
+    if (computerColorDialogLoader.status === Loader.Null)
         computerColorDialogLoader.sourceComponent =
                 computerColorDialogComponent
     var dialog = computerColorDialogLoader.item
@@ -313,18 +263,18 @@ function showComputerColorDialog() {
     dialog.computerPlays1 = computerPlays1
     dialog.computerPlays2 = computerPlays2
     dialog.computerPlays3 = computerPlays3
-    dialog.visible = true
+    dialog.open()
 }
 
 function showGameOver() {
-    if (! boardModel.isGameOver)
+    if (! gameModel.isGameOver)
         return
     var msg, points0, points1, points2, points3
-    switch (boardModel.gameVariant) {
+    switch (gameModel.gameVariant) {
     case "duo":
     case "junior":
-        points0 = boardModel.points0
-        points1 = boardModel.points1
+        points0 = gameModel.points0
+        points1 = gameModel.points1
         if (points0 > points1)
             msg = qsTr("Blue wins.")
         else if (points0 < points1)
@@ -334,8 +284,8 @@ function showGameOver() {
         break
     case "classic_2":
     case "trigon_2":
-        points0 = boardModel.points0 + boardModel.points2
-        points1 = boardModel.points1 + boardModel.points3
+        points0 = gameModel.points0 + gameModel.points2
+        points1 = gameModel.points1 + gameModel.points3
         if (points0 > points1)
             msg = qsTr("Blue/Red wins.")
         else if (points0 < points1)
@@ -345,9 +295,9 @@ function showGameOver() {
         break
     case "classic_3":
     case "trigon_3":
-        points0 = boardModel.points0
-        points1 = boardModel.points1
-        points2 = boardModel.points2
+        points0 = gameModel.points0
+        points1 = gameModel.points1
+        points2 = gameModel.points2
         var maxPoints = Math.max(points0, points1, points2)
         var nuWinners = 0
         if (points0 === maxPoints) ++nuWinners
@@ -363,10 +313,10 @@ function showGameOver() {
             msg = qsTr("Red wins.")
         break
     default:
-        points0 = boardModel.points0
-        points1 = boardModel.points1
-        points2 = boardModel.points2
-        points3 = boardModel.points3
+        points0 = gameModel.points0
+        points1 = gameModel.points1
+        points2 = gameModel.points2
+        points3 = gameModel.points3
         maxPoints = Math.max(points0, points1, points2, points3)
         nuWinners = 0
         if (points0 === maxPoints) ++nuWinners
@@ -384,34 +334,55 @@ function showGameOver() {
         else if (points3 === maxPoints)
             msg = qsTr("Green wins.")
     }
-    gameDisplay.showMessage(msg)
+    showInfo(msg)
 }
 
-function showMessageDialog(text, acceptedFunc) {
-    if (messageDialogLoader.status == Loader.Null)
-        messageDialogLoader.sourceComponent = messageDialogComponent
-    var dialog = messageDialogLoader.item
+function showError(text) {
+    if (errorMessageLoader.status === Loader.Null)
+        errorMessageLoader.sourceComponent = errorMessageComponent
+    var dialog = errorMessageLoader.item
+    dialog.text = text
+    dialog.open()
+}
+
+function showInfo(text) {
+    if (infoMessageLoader.status === Loader.Null)
+        infoMessageLoader.sourceComponent = infoMessageComponent
+    var dialog = infoMessageLoader.item
+    dialog.text = text
+    dialog.open()
+}
+
+function showQuestion(text, acceptedFunc) {
+    if (questionMessageLoader.status === Loader.Null)
+        questionMessageLoader.sourceComponent = questionMessageComponent
+    var dialog = questionMessageLoader.item
     dialog.text = text
     dialog.accepted.connect(acceptedFunc)
-    dialog.visible = true
+    dialog.open()
+}
+
+function truncate() {
+    showQuestion(qsTr("Truncate this subtree?"),
+                 function() { gameModel.truncate() })
+}
+
+function truncateChildren() {
+    showQuestion(qsTr("Truncate children?"),
+                 function() { gameModel.truncateChildren() })
 }
 
 function undo() {
-    if (! boardModel.canUndo)
+    if (! gameModel.canUndo)
         return
     cancelGenMove()
-    gameDisplay.clearMessage()
-    // Immediately show the last color to move, otherwise the piece movement
-    // animation will target a part of the piece selector that is currently not
-    // visible
-    gameDisplay.showPiecesImmediately(boardModel.getLastMoveColor())
-    boardModel.undo()
-    clearMarks()
-    markLastMove()
-    gameDisplay.pickedPiece = null
+    gameDisplay.transitionsEnabled = true
+    gameModel.undo()
+    gameDisplay.transitionsEnabled = false
+    gameDisplay.dropPiece()
 }
 
 function quit() {
     cancelGenMove()
-    autosave()
+    gameModel.autoSave()
 }
