@@ -16,7 +16,6 @@
 #include "libpentobi_base/ScoreUtil.h"
 
 using libboardgame_sgf::Writer;
-using libboardgame_util::log;
 using libboardgame_util::trim;
 using libpentobi_base::get_multiplayer_result;
 using libpentobi_base::Move;
@@ -60,7 +59,7 @@ float TwoGtp::get_result(unsigned player_black)
     auto nu_players = m_bd.get_nu_players();
     if (nu_players == 2)
     {
-        int score = m_bd.get_score(Color(0));
+        int score = m_bd.get_score_twoplayer(Color(0));
         if (score > 0)
             result = 1;
         else if (score < 0)
@@ -85,11 +84,9 @@ float TwoGtp::get_result(unsigned player_black)
 void TwoGtp::play_game(unsigned game_number)
 {
     if (! m_quiet)
-    {
-        log("=====================================================");
-        log("Game ", game_number);
-        log("=====================================================");
-    }
+        LIBBOARDGAME_LOG("================================================\n"
+                         "Game ", game_number, "\n"
+                         "================================================");
     m_bd.init();
     send_both("clear_board");
     auto cpu_black = send_cputime(m_black);
@@ -123,7 +120,7 @@ void TwoGtp::play_game(unsigned game_number)
                                                     m_bd, to_play, mv))
         {
             is_real_move[m_bd.get_nu_moves()] = false;
-            log("Playing fast opening move");
+            LIBBOARDGAME_LOG("Playing fast opening move");
             player_connection.send("play " + color + " " + m_bd.to_string(mv));
         }
         else
@@ -138,7 +135,8 @@ void TwoGtp::play_game(unsigned game_number)
             mv = m_bd.from_string(response);
         }
         sgf.begin_node();
-        sgf.write_property(string(1, toupper(color[0])), m_bd.to_string(mv));
+        sgf.write_property(string(1, static_cast<char>(toupper(color[0]))),
+                           m_bd.to_string(mv));
         sgf.end_node();
         if (mv.is_null() || ! m_bd.is_legal(to_play, mv))
             throw runtime_error("invalid move: " + m_bd.to_string(mv));
