@@ -9,11 +9,11 @@
 
 #include <array>
 #include "GtpConnection.h"
-#include "OutputFile.h"
-#include "libpentobi_base/Color.h"
-#include "libpentobi_base/Variant.h"
+#include "Output.h"
+#include "libpentobi_base/Board.h"
 
 using namespace std;
+using libpentobi_base::Board;
 using libpentobi_base::Color;
 using libpentobi_base::Variant;
 
@@ -23,29 +23,31 @@ class TwoGtp
 {
 public:
     TwoGtp(const string& black, const string& white, Variant variant,
-           unsigned nu_games, OutputFile& output, bool quiet,
-           const string& log_prefix);
+           unsigned nu_games, Output& output, bool quiet,
+           const string& log_prefix, bool fast_open);
 
     void run();
 
 private:
     bool m_quiet;
 
+    bool m_fast_open;
+
     Variant m_variant;
 
     unsigned m_nu_games;
 
-    OutputFile& m_output;
+    Board m_bd;
+
+    Output& m_output;
 
     GtpConnection m_black;
 
     GtpConnection m_white;
 
-    array<bool, Color::range> m_has_moves;
+    array<string, Color::range> m_colors;
 
-    array<const char*, Color::range> m_colors;
-
-    float get_result(unsigned player_black, const string& final_score);
+    float get_result(unsigned player_black);
 
     void play_game(unsigned game_number);
 
