@@ -34,11 +34,11 @@ LIBBOARDGAME_TEST_CASE(pentobi_base_game_current_defined_invalid_root)
     {
         game.init(root);
     }
-    catch (const Exception&)
+    catch (const runtime_error&)
     {
         // ignore
     }
-    LIBBOARDGAME_CHECK_EQUAL(&game.get_current(), &game.get_tree().get_root());
+    LIBBOARDGAME_CHECK_EQUAL(&game.get_current(), &game.get_root());
 }
 
 //-----------------------------------------------------------------------------
